"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const path = require("path");
const async = require("async");
const inquirer = require("inquirer");
const ora = require("ora");
const pluralize = require("pluralize");
const config_1 = require("../config");
const sonarwhal_1 = require("../sonarwhal");
const types_1 = require("../types");
const debug_1 = require("../utils/debug");
const get_as_uri_1 = require("../utils/get-as-uri");
const logger = require("../utils/logging");
const misc_1 = require("../utils/misc");
const resourceLoader = require("../utils/resource-loader");
const npm_1 = require("../utils/npm");
const init_1 = require("./wizards/init");
const each = util_1.promisify(async.each);
const debug = debug_1.debug(__filename);
const confirmLaunchInit = () => {
    debug(`Initiating launch init confirm.`);
    const question = [{
            message: `A valid configuration file can't be found. Do you want to create a new one?`,
            name: 'confirm',
            type: 'confirm'
        }];
    return inquirer.prompt(question);
};
const askUserToCreateConfig = async () => {
    const launchInit = await confirmLaunchInit();
    if (!launchInit.confirm) {
        return false;
    }
    await init_1.initSonarwhalrc({ init: true });
    logger.log(`Configuration file .sonarwhalrc was created.`);
    return true;
};
const askUserToInstallDependencies = async (dependencies) => {
    const question = [{
            message: `There ${pluralize('is', dependencies.length)} ${dependencies.length} ${pluralize('package', dependencies.length)} from your .sonarwhalrc file not installed or with an incompatible version. Do you want us to try to install/update them?`,
            name: 'confirm',
            type: 'confirm'
        }];
    const answer = await inquirer.prompt(question);
    return answer.confirm;
};
const getUserConfig = (actions) => {
    let config;
    const configPath = actions.config || config_1.SonarwhalConfig.getFilenameForDirectory(process.cwd());
    if (!configPath) {
        return null;
    }
    debug(`Loading configuration file from ${configPath}.`);
    try {
        const resolvedPath = path.resolve(process.cwd(), configPath);
        config = config_1.SonarwhalConfig.loadConfigFile(resolvedPath);
    }
    catch (e) {
        logger.error(e);
        config = null;
    }
    return config;
};
const messages = {
    'fetch::end': '%url% downloaded',
    'fetch::start': 'Downloading %url%',
    'scan::end': 'Finishing...',
    'scan::start': 'Analyzing %url%',
    'traverse::down': 'Traversing the DOM',
    'traverse::end': 'Traversing finished',
    'traverse::start': 'Traversing the DOM',
    'traverse::up': 'Traversing the DOM'
};
const getEvent = (event) => {
    if (event.startsWith('fetch::end')) {
        return 'fetch::end';
    }
    return event;
};
const setUpUserFeedback = (sonarwhalInstance, spinner) => {
    sonarwhalInstance.prependAny((event, value) => {
        const message = messages[getEvent(event)];
        if (!message) {
            return;
        }
        spinner.text = message.replace('%url%', misc_1.cutString(value.resource));
    });
};
exports.sonarwhal = null;
exports.analyze = async (actions) => {
    if (!actions._) {
        return false;
    }
    const targets = get_as_uri_1.getAsUris(actions._);
    if (targets.length === 0) {
        return false;
    }
    let userConfig = await getUserConfig(actions);
    if (!userConfig) {
        logger.error(`Couldn't find a valid path to load the configuration file.`);
        const created = await askUserToCreateConfig();
        if (created) {
            userConfig = await getUserConfig(actions);
        }
        else {
            logger.error(`Unable to find a valid configuration file. Please add a .sonarwhalrc file by running 'sonarwhal --init'. `);
            return false;
        }
    }
    const config = config_1.SonarwhalConfig.fromConfig(userConfig, actions);
    const resources = resourceLoader.loadResources(config);
    if (resources.missing.length > 0 || resources.incompatible.length > 0) {
        const missingPackages = resources.missing.map((name) => {
            return `@sonarwhal/${name}`;
        });
        const incompatiblePackages = resources.incompatible.map((name) => {
            return `@sonarwhal/${name}`;
        });
        if (!(await askUserToInstallDependencies(resources.missing.concat(resources.incompatible)) &&
            await npm_1.installPackages(missingPackages) &&
            await npm_1.installPackages(incompatiblePackages))) {
            return false;
        }
    }
    const invalidConfigRules = config_1.SonarwhalConfig.validateRulesConfig(config).invalid;
    if (invalidConfigRules.length > 0) {
        logger.error(`Invalid rule configuration in .sonarwhalrc: ${invalidConfigRules.join(', ')}.`);
        return false;
    }
    exports.sonarwhal = new sonarwhal_1.Sonarwhal(config, resources);
    const start = Date.now();
    const spinner = ora({ spinner: 'line' });
    let exitCode = 0;
    if (!actions.debug) {
        spinner.start();
        setUpUserFeedback(exports.sonarwhal, spinner);
    }
    const endSpinner = (method) => {
        if (!actions.debug && spinner[method]) {
            spinner[method]();
        }
    };
    const hasError = (reports) => {
        return reports.some((result) => {
            return result.severity === types_1.Severity.error;
        });
    };
    const print = async (reports, target) => {
        if (hasError(reports)) {
            endSpinner('fail');
        }
        else {
            endSpinner('succeed');
        }
        await each(exports.sonarwhal.formatters, async (formatter) => {
            await formatter.format(reports, target);
        });
    };
    exports.sonarwhal.on('print', print);
    for (const target of targets) {
        try {
            const results = await exports.sonarwhal.executeOn(target);
            if (hasError(results)) {
                exitCode = 1;
            }
            await print(results, target.href);
        }
        catch (e) {
            exitCode = 1;
            endSpinner('fail');
            debug(`Failed to analyze: ${target.href}`);
            debug(e);
        }
    }
    await exports.sonarwhal.close();
    debug(`Total runtime: ${Date.now() - start}ms`);
    return exitCode === 0;
};
