"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util_1 = require("util");
const inquirer = require("inquirer");
const debug_1 = require("../../utils/debug");
const logger = require("../../utils/logging");
const resource_loader_1 = require("../../utils/resource-loader");
const resourcetype_1 = require("../../enums/resourcetype");
const browserslist_1 = require("../browserslist");
const npm_1 = require("../../utils/npm");
const debug = debug_1.debug(__filename);
const defaultFormatter = 'summary';
const anyResources = (resources, type) => {
    if (resources.length > 0) {
        return true;
    }
    logger.error(`Couldn't find any installed ${type}s. Visit https://www.npmjs.com/search?q=%40sonarwhal%2F${type}.`);
    return false;
};
const getConfigurationName = (pkgName) => {
    const nameSplitted = pkgName.split('/');
    return nameSplitted[1].replace('configuration-', '');
};
const extendConfig = async () => {
    const configPackages = await npm_1.getOfficialPackages(resourcetype_1.ResourceType.configuration);
    if (!anyResources(configPackages, resourcetype_1.ResourceType.configuration)) {
        return null;
    }
    const choices = configPackages.map((pkg) => {
        return {
            name: getConfigurationName(pkg.name),
            value: pkg.name
        };
    });
    const questions = [{
            choices,
            message: 'Choose the configuration you want to extend from',
            name: 'configuration',
            pageSize: 15,
            type: 'list'
        }];
    const answers = await inquirer.prompt(questions);
    const sonarwhalConfig = { extends: [getConfigurationName(answers.configuration)] };
    return {
        config: sonarwhalConfig,
        packages: [answers.configuration]
    };
};
const customConfig = async () => {
    const connectorKeys = resource_loader_1.getInstalledResources(resourcetype_1.ResourceType.connector).concat(resource_loader_1.getCoreResources(resourcetype_1.ResourceType.connector));
    const formattersKeys = resource_loader_1.getInstalledResources(resourcetype_1.ResourceType.formatter).concat(resource_loader_1.getCoreResources(resourcetype_1.ResourceType.formatter));
    const parsersKeys = resource_loader_1.getInstalledResources(resourcetype_1.ResourceType.parser).concat(resource_loader_1.getCoreResources(resourcetype_1.ResourceType.parser));
    const rulesKeys = resource_loader_1.getInstalledResources(resourcetype_1.ResourceType.rule).concat(resource_loader_1.getCoreResources(resourcetype_1.ResourceType.rule));
    if (!anyResources(connectorKeys, resourcetype_1.ResourceType.connector) ||
        !anyResources(formattersKeys, resourcetype_1.ResourceType.formatter) ||
        !anyResources(rulesKeys, resourcetype_1.ResourceType.rule)) {
        return null;
    }
    const customQuestions = [
        {
            choices: connectorKeys,
            message: 'What connector do you want to use?',
            name: 'connector',
            type: 'list'
        },
        {
            choices: formattersKeys,
            default: defaultFormatter,
            message: 'What formatter do you want to use?',
            name: 'formatter',
            type: 'list'
        },
        {
            choices: rulesKeys,
            message: 'Choose the rules you want to add to your configuration',
            name: 'rules',
            pageSize: 15,
            type: 'checkbox',
            when: (answers) => {
                return !answers.default;
            }
        }
    ];
    if (parsersKeys.length > 0) {
        customQuestions.push({
            choices: parsersKeys,
            message: 'What parsers do you want to use?',
            name: 'parsers',
            pageSize: 15,
            type: 'checkbox'
        });
    }
    const results = await inquirer.prompt(customQuestions);
    const sonarwhalConfig = {
        browserslist: [],
        connector: {
            name: '',
            options: { waitFor: 1000 }
        },
        extends: [],
        formatters: [defaultFormatter],
        ignoredUrls: [],
        rules: {},
        rulesTimeout: 120000
    };
    sonarwhalConfig.connector.name = results.connector;
    sonarwhalConfig.formatters = [results.formatter];
    results.rules.forEach((rule) => {
        sonarwhalConfig.rules[rule] = 'error';
    });
    sonarwhalConfig.browserslist = await browserslist_1.generateBrowserslistConfig();
    return { config: sonarwhalConfig };
};
exports.initSonarwhalrc = async (options) => {
    if (!options.init) {
        return false;
    }
    debug('Starting --init');
    logger.log('Welcome to sonarwhal configuration generator');
    const initialQuestion = [{
            choices: ['predefined', 'custom'],
            default: 'predefined',
            message: 'Do you want to use a predefined configuration or create your own based on your installed packages?',
            name: 'configType',
            type: 'list'
        }];
    const initialAnswer = await inquirer.prompt(initialQuestion);
    const result = initialAnswer.configType === 'predefined' ?
        await extendConfig() :
        await customConfig();
    if (!result) {
        return false;
    }
    const filePath = path.join(process.cwd(), '.sonarwhalrc');
    await util_1.promisify(fs.writeFile)(filePath, JSON.stringify(result.config, null, 4), 'utf8');
    if (Array.isArray(result.packages) && result.packages.length > 0) {
        const isInstalled = resource_loader_1.getInstalledResources(resourcetype_1.ResourceType.configuration).includes(getConfigurationName(result.packages[0]));
        if (isInstalled) {
            return true;
        }
        await npm_1.installPackages(result.packages);
    }
    return true;
};
