"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const util_1 = require("util");
const fs = require("fs-extra");
const inquirer = require("inquirer");
const mkdirp = require("mkdirp");
const logger = require("../../utils/logging");
const misc_1 = require("../../utils/misc");
const handlebars_1 = require("../../utils/handlebars");
const events = {
    'element::': 'ElementFound',
    'fetch::end::*': 'FetchEnd',
    'fetch::end::html': 'FetchEnd',
    'fetch::error': 'FetchError',
    'fetch::start': 'FetchStart',
    'scan::end': 'ScanEnd',
    'scan::start': 'ScanStart',
    'traverse::down': 'TraverseDown',
    'traverse::end': 'TraverseEnd',
    'traverse::start': 'TraverseStart',
    'traverse::up': 'TraverseUp'
};
const capitalize = (str) => {
    if (str === '*') {
        return '';
    }
    const splittedText = misc_1.normalizeStringByDelimiter(str, ' ').split(' ');
    const result = splittedText.reduce((total, text) => {
        if (!text) {
            return total;
        }
        return `${total}${text.charAt(0).toUpperCase()}${text.slice(1)}`;
    }, '');
    return result;
};
class NewParser {
    constructor(parserData) {
        this.events = [];
        this.isParser = true;
        this.name = parserData.name;
        this.normalizedName = misc_1.normalizeStringByDelimiter(parserData.name, '-');
        this.capitalizedName = capitalize(parserData.name);
        this.description = handlebars_1.escapeSafeString(parserData.description);
        const eventTypesSet = new Set();
        parserData.eventsSelected.forEach((event) => {
            const isElement = event.event === 'element::';
            const type = events[event.event];
            const eventSplit = event.event.split('::');
            const handler = `on${capitalize(eventSplit[0])}${capitalize(isElement ? event.element : eventSplit[1])}${eventSplit[2] ? capitalize(eventSplit[2]) : ''}`;
            const varName = `${type.charAt(0).toLowerCase()}${type.substr(1)}`;
            this.events.push({
                event: isElement ? event.event + event.element : event.event,
                handler,
                type,
                varName
            });
            eventTypesSet.add(type);
        });
        this.official = parserData.isOfficial;
        const prefix = this.official ? '@sonarwhal/' : 'sonarwhal-';
        this.version = handlebars_1.sonarwhalPackage.version;
        this.packageMain = `dist/src/index.js`;
        this.packageName = `${prefix}parser-${this.normalizedName}`;
        this.destination = path.join(process.cwd(), `parser-${this.normalizedName}`);
        this.processDir = process.cwd();
        this.eventTypes = [...eventTypesSet];
    }
}
const mkdirpAsync = util_1.promisify(mkdirp);
const eventList = Object.keys(events);
const TEMPLATE_PATH = './templates/new-parser';
const SHARED_TEMPLATE_PATH = './shared-templates';
const questions = (repeat = false) => {
    return [
        {
            default: 'newParser',
            message: `What's the name of this new parser?`,
            name: 'name',
            type: 'input',
            when() {
                return !repeat;
            }
        },
        {
            default(answers) {
                return `Description for ${answers.name}`;
            },
            message(answers) {
                return `What's the description of this new parser '${answers.name}'?`;
            },
            name: 'description',
            type: 'input',
            when() {
                return !repeat;
            }
        },
        {
            choices: eventList,
            default: 'fetch::end::*',
            message: 'Please select an event to subscribe in the parser',
            name: 'event',
            type: 'list',
            when() {
                return repeat;
            }
        },
        {
            default: 'div',
            message: `What's the element for the event?`,
            name: 'element',
            type: 'input',
            when(answers) {
                return answers.event === 'element::';
            }
        },
        {
            default: true,
            message: 'Want to add more events (yes)?',
            name: 'again',
            type: 'confirm',
            when() {
                return repeat;
            }
        }
    ];
};
const copyFiles = async (data) => {
    const files = path.join(__dirname, 'files');
    const noOfficialFiles = path.join(__dirname, 'no-official-files');
    logger.log(`Creating new rule in ${data.destination}`);
    if (!data.official) {
        await fs.copy(noOfficialFiles, data.destination);
    }
    await fs.copy(files, data.destination);
    logger.log('Files copied');
};
const generateFiles = async (data) => {
    const files = [
        {
            destination: path.join(data.destination, 'src', `${data.normalizedName}.ts`),
            path: path.join(__dirname, TEMPLATE_PATH, 'script.hbs')
        },
        {
            destination: path.join(data.destination, 'tests', `${data.normalizedName}.ts`),
            path: path.join(__dirname, TEMPLATE_PATH, 'tests.hbs')
        },
        {
            destination: path.join(data.destination, 'src', `index.ts`),
            path: path.join(__dirname, TEMPLATE_PATH, 'index.hbs')
        },
        {
            destination: path.join(data.destination, 'README.md'),
            path: path.join(__dirname, TEMPLATE_PATH, 'doc.hbs')
        },
        {
            destination: path.join(data.destination, 'tsconfig.json'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'tsconfig.json.hbs')
        },
        {
            destination: path.join(data.destination, 'package.json'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'package.hbs')
        }
    ];
    if (!data.official) {
        files.push({
            destination: path.join(data.destination, '.sonarwhalrc'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'config.hbs')
        });
    }
    for (const file of files) {
        const { destination: dest, path: filePath } = file;
        const fileContent = await handlebars_1.compileTemplate(filePath, data);
        await mkdirpAsync(path.dirname(dest));
        await misc_1.writeFileAsync(dest, fileContent);
    }
};
exports.newParser = async (actions) => {
    if (!actions.newParser) {
        return false;
    }
    const results = await inquirer.prompt(questions());
    const eventsSelected = [];
    results.isOfficial = await misc_1.isOfficial();
    if (!results.name) {
        throw new Error(`Name can't be empty`);
    }
    const askEvents = async () => {
        const event = await inquirer.prompt(questions(true));
        if (event.event !== 'element::') {
            eventList.splice(eventList.indexOf(event.event), 1);
            eventsSelected.push(event);
        }
        else {
            const elementExists = eventsSelected.find((ev) => {
                return ev.element === event.element;
            });
            if (!elementExists) {
                eventsSelected.push(event);
            }
        }
        if (event.again) {
            await askEvents();
        }
        return;
    };
    await askEvents();
    results.eventsSelected = eventsSelected;
    const parserData = new NewParser(results);
    await copyFiles(parserData);
    await generateFiles(parserData);
    logger.log(`
New parser created in ${parserData.destination}

--------------------------------------
----          How to use          ----
--------------------------------------`);
    if (parserData.official) {
        logger.log(`1. Run 'yarn' to install the dependencies.
2. Go to the folder '${parserData.destination}'.
3. Run 'yarn build' to build the project.
4. Go to the folder 'packages/sonarwhal'.
5. Add your parser to '.sonarwhalrc'.
6. Run 'yarn sonarwhal https://YourUrl' to analyze your site.`);
    }
    else {
        logger.log(`1. Go to the folder '${parserData.destination}'.
2. Run 'npm run init' to install all the dependencies and build the project.
3. Run 'npm run sonarwhal -- https://YourUrl' to analyze you site.`);
    }
    return true;
};
