"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const util_1 = require("util");
const Handlebars = require("handlebars");
const fs = require("fs-extra");
const inquirer = require("inquirer");
const mkdirp = require("mkdirp");
const category_1 = require("../../enums/category");
const rulescope_1 = require("../../enums/rulescope");
const logger = require("../../utils/logging");
const misc_1 = require("../../utils/misc");
const handlebars_1 = require("../../utils/handlebars");
const events = new Map([
    ['dom', ['ElementFound']],
    ['request', ['FetchStart', 'FetchEnd', 'FetchError']],
    ['thirdPartyService', ['FetchStart', 'FetchEnd']],
    ['jsInjection', ['ScanEnd']]
]);
var QuestionsType;
(function (QuestionsType) {
    QuestionsType["main"] = "main";
    QuestionsType["rule"] = "rule";
})(QuestionsType = exports.QuestionsType || (exports.QuestionsType = {}));
const getEventsByUseCase = (useCase) => {
    return events.get(useCase).join(', ');
};
class NewRule {
    constructor(ruleData, parentName) {
        this.isRule = true;
        this.name = ruleData.name;
        this.normalizedName = misc_1.normalizeStringByDelimiter(ruleData.name, '-');
        this.className = `${parentName ? misc_1.toPascalCase(parentName) : ''}${misc_1.toPascalCase(this.normalizedName)}Rule`;
        this.category = ruleData.category || category_1.Category.other;
        this.description = handlebars_1.escapeSafeString(ruleData.description);
        this.elementType = ruleData.elementType;
        this.events = getEventsByUseCase(ruleData.useCase);
        this.useCase = {
            dom: false,
            jsInjection: false,
            request: false,
            thirdPartyService: false
        };
        this.useCase[ruleData.useCase] = true;
        this.scope = ruleData.scope;
        this.parentName = parentName;
    }
}
class RulePackage {
    constructor(data) {
        this.isRule = true;
        this.name = data.name;
        this.isMulti = data.multi;
        this.normalizedName = misc_1.normalizeStringByDelimiter(data.name, '-');
        this.description = handlebars_1.escapeSafeString(data.description);
        this.official = data.official;
        this.packageMain = `dist/src/index.js`;
        const prefix = this.official ? '@sonarwhal/' : 'sonarwhal-';
        this.packageName = `${prefix}rule-${this.normalizedName}`;
        this.version = handlebars_1.sonarwhalPackage.version;
        this.rules = [];
        if (this.isMulti) {
            data.rules.forEach((rule) => {
                this.rules.push(new NewRule(rule, this.normalizedName));
            });
        }
        else {
            this.rules.push(new NewRule(data));
        }
        this.destination = path.join(process.cwd(), `rule-${this.normalizedName}`);
    }
}
const mkdirpAsync = util_1.promisify(mkdirp);
const TEMPLATE_PATH = './templates/new-rule';
const SHARED_TEMPLATE_PATH = './shared-templates';
const partialEventCode = misc_1.readFile(path.join(__dirname, 'templates', 'new-rule', 'partial-event-code.hbs'));
Handlebars.registerPartial('event-code', partialEventCode);
Handlebars.registerHelper('toCamelCase', misc_1.toCamelCase);
const categories = [];
for (const [, value] of Object.entries(category_1.Category)) {
    if (value !== 'other') {
        categories.push({ name: value });
    }
}
const scopes = [];
for (const [, value] of Object.entries(rulescope_1.RuleScope)) {
    if (value !== 'other') {
        scopes.push({ name: value });
    }
}
const useCases = [
    {
        name: 'DOM',
        value: 'dom'
    },
    {
        name: 'Resource Request',
        value: 'request'
    },
    {
        name: 'Third Party Service',
        value: 'thirdPartyService'
    },
    {
        name: 'JS injection',
        value: 'jsInjection'
    }
];
exports.questions = (type) => {
    const notEmpty = (value) => {
        return value.trim() !== '';
    };
    return [{
            message: `Is this a package with multiple rules? (yes)`,
            name: 'multi',
            type: 'confirm',
            when: () => {
                return type === QuestionsType.main;
            }
        },
        {
            default(answers) {
                return answers.multi ? 'newPackage' : 'newRule';
            },
            message(answers) {
                return `What's the name of this new ${answers.multi ? 'package' : 'rule'}?`;
            },
            name: 'name',
            type: 'input',
            validate: notEmpty
        },
        {
            default(answers) {
                return `Description for ${answers.name}`;
            },
            message(answers) {
                return `What's the description of this new ${answers.multi ? 'package' : 'rule'} '${answers.name}'?`;
            },
            name: 'description',
            type: 'input',
            validate: notEmpty
        },
        {
            choices: categories,
            default: category_1.Category.interoperability,
            message: 'Please select the category of this new rule:',
            name: 'category',
            type: 'list',
            when(answers) {
                return !answers.multi;
            }
        },
        {
            choices: useCases,
            default: 'dom',
            message: 'Please select the category of use case:',
            name: 'useCase',
            type: 'list',
            when(answers) {
                return !answers.multi;
            }
        },
        {
            default: 'div',
            message: 'What DOM element does the rule need access to?',
            name: 'elementType',
            type: 'input',
            validate: notEmpty,
            when: (answers) => {
                return answers.useCase === 'dom';
            }
        },
        {
            choices: scopes,
            default: rulescope_1.RuleScope.any,
            message: 'Please select the scope of this new rule:',
            name: 'scope',
            type: 'list',
            when(answers) {
                return !answers.multi;
            }
        },
        {
            default: true,
            message: 'Want to add more rules (yes)?',
            name: 'again',
            type: 'confirm',
            when: () => {
                return type === QuestionsType.rule;
            }
        }];
};
const copyFiles = async (origin, destination) => {
    logger.log(`Creating new rule in ${destination}`);
    await fs.copy(origin, destination);
    logger.log('External files copied');
};
const generateRuleFiles = async (destination, data) => {
    const commonFiles = [
        {
            destination: path.join(destination, 'src', `index.ts`),
            path: path.join(__dirname, TEMPLATE_PATH, 'index.ts.hbs')
        },
        {
            destination: path.join(destination, 'README.md'),
            path: path.join(__dirname, TEMPLATE_PATH, 'readme.md.hbs')
        },
        {
            destination: path.join(destination, 'tsconfig.json'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'tsconfig.json.hbs')
        },
        {
            destination: path.join(destination, 'package.json'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'package.hbs')
        }
    ];
    if (!data.official) {
        commonFiles.push({
            destination: path.join(destination, '.sonarwhalrc'),
            path: path.join(__dirname, SHARED_TEMPLATE_PATH, 'config.hbs')
        });
    }
    const ruleFile = {
        destination: path.join(destination, 'src'),
        path: path.join(__dirname, TEMPLATE_PATH, 'rule.ts.hbs')
    };
    const testFile = {
        destination: path.join(destination, 'tests'),
        path: path.join(__dirname, TEMPLATE_PATH, 'tests.ts.hbs')
    };
    const docFile = {
        destination: path.join(destination, 'docs'),
        path: path.join(__dirname, TEMPLATE_PATH, 'rule-doc.hbs')
    };
    for (const file of commonFiles) {
        const { destination: dest, path: p } = file;
        const fileContent = await handlebars_1.compileTemplate(p, data);
        await mkdirpAsync(path.dirname(dest));
        await misc_1.writeFileAsync(dest, fileContent);
    }
    for (const rule of data.rules) {
        const [ruleContent, testContent] = await Promise.all([handlebars_1.compileTemplate(ruleFile.path, rule), handlebars_1.compileTemplate(testFile.path, rule)]);
        const rulePath = path.join(ruleFile.destination, `${rule.normalizedName}.ts`);
        const testPath = path.join(testFile.destination, `${rule.normalizedName}.ts`);
        const docPath = path.join(docFile.destination, `${rule.normalizedName}.md`);
        await Promise.all([mkdirpAsync(path.dirname(rulePath)), mkdirpAsync(path.dirname(testPath))]);
        await Promise.all([misc_1.writeFileAsync(rulePath, ruleContent), misc_1.writeFileAsync(testPath, testContent)]);
        if (data.isMulti) {
            const docContent = await handlebars_1.compileTemplate(docFile.path, rule);
            await mkdirpAsync(path.dirname(docPath));
            await misc_1.writeFileAsync(docPath, docContent);
        }
    }
};
exports.newRule = async (actions) => {
    if (!actions.newRule) {
        return false;
    }
    try {
        const results = await inquirer.prompt(exports.questions(QuestionsType.main));
        const rules = [];
        results.official = await misc_1.isOfficial();
        const askRules = async () => {
            const rule = await inquirer.prompt(exports.questions(QuestionsType.rule));
            rules.push(rule);
            if (rule.again) {
                await askRules();
            }
        };
        if (results.multi) {
            await askRules();
        }
        results.rules = rules;
        const rulePackage = new RulePackage(results);
        const noOfficialOrigin = path.join(__dirname, 'no-official-files');
        const files = path.join(__dirname, 'files');
        if (!rulePackage.official) {
            await copyFiles(noOfficialOrigin, rulePackage.destination);
        }
        await copyFiles(files, rulePackage.destination);
        await generateRuleFiles(rulePackage.destination, rulePackage);
        logger.log(`
New ${rulePackage.isMulti ? 'package' : 'rule'} ${rulePackage.name} created in ${rulePackage.destination}

--------------------------------------
----          How to use          ----
--------------------------------------`);
        if (rulePackage.official) {
            logger.log(`1. Run 'yarn' to install the dependencies.
2. Go to the folder 'packages/rule-${rulePackage.normalizedName}'.
3. Run 'yarn build' to build the project.
4. Go to the folder 'packages/sonarwhal'.
5. Add your rule to '.sonarwhalrc'.
6. Run 'yarn sonarwhal https://YourUrl' to analyze your site.`);
        }
        else {
            logger.log(`1. Go to the folder 'rule-${rulePackage.normalizedName}'.
2. Run 'npm run init' to install all the dependencies and build the project.
3. Run 'npm run sonarwhal -- https://YourUrl' to analyze you site.`);
        }
        return true;
    }
    catch (e) {
        logger.error('Error trying to create new rule');
        logger.error(e);
        return false;
    }
};
