"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const path = require("path");
const browserslist = require("browserslist");
const shell = require("shelljs");
const _ = require("lodash");
const debug_1 = require("./utils/debug");
const misc_1 = require("./utils/misc");
const config_validator_1 = require("./config/config-validator");
const normalize_rules_1 = require("./config/normalize-rules");
const config_rules_1 = require("./config/config-rules");
const resourceLoader = require("./utils/resource-loader");
const debug = debug_1.debug(__filename);
const CONFIG_FILES = [
    '.sonarwhalrc',
    '.sonarwhalrc.js',
    '.sonarwhalrc.json',
    'package.json'
];
const loadPackageJSONConfigFile = (filePath) => {
    debug(`Loading package.json config file: ${filePath}`);
    try {
        return misc_1.loadJSONFile(filePath).sonarwhalConfig || null;
    }
    catch (e) {
        debug(`Error reading package.json file: ${filePath}`);
        e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
        throw e;
    }
};
const composeConfig = (userConfig) => {
    debug('Composing configuration from extends');
    if (!userConfig.extends || !Array.isArray(userConfig.extends) || userConfig.extends.length === 0) {
        return userConfig;
    }
    const configurations = userConfig.extends.map((config) => {
        const loadedConfiguration = resourceLoader.loadConfiguration(config);
        if (!config_validator_1.validateConfig(loadedConfiguration)) {
            throw new Error(`Configuration package "${config}" is not valid`);
        }
        return composeConfig(loadedConfiguration);
    });
    const finalConfig = _.merge({}, ...configurations, userConfig);
    return finalConfig;
};
const loadIgnoredUrls = (userConfig) => {
    debug('Initializing ignored urls');
    const ignoredUrls = new Map();
    if (userConfig.ignoredUrls) {
        userConfig.ignoredUrls.forEach((ignoredUrl) => {
            const { domain: urlRegexString, rules } = ignoredUrl;
            rules.forEach((rule) => {
                const ruleName = rule === '*' ? 'all' : rule;
                const urlsInRule = ignoredUrls.get(ruleName);
                const urlRegex = new RegExp(urlRegexString, 'i');
                if (!urlsInRule) {
                    ignoredUrls.set(ruleName, [urlRegex]);
                }
                else {
                    urlsInRule.push(urlRegex);
                }
            });
        });
    }
    return ignoredUrls;
};
const buildRulesConfigFromRuleNames = (ruleNames, severity) => {
    const ruleConfig = {};
    for (const ruleName of ruleNames) {
        ruleConfig[ruleName] = severity;
    }
    return ruleConfig;
};
const updateConfigWithCommandLineValues = (config, actions) => {
    debug('overriding config settings with values provided via CLI');
    if (actions && actions.formatters) {
        config.formatters = actions.formatters.split(',');
        debug(`Using formatters option provided from command line: ${actions.formatters}`);
    }
    if (actions && actions.rules) {
        const ruleNames = actions.rules.split(',');
        config.rules = buildRulesConfigFromRuleNames(ruleNames, 'error');
        debug(`Using rules option provided from command line: ${actions.rules}`);
    }
};
class SonarwhalConfig {
    constructor(userConfig, browsers, ignoredUrls, rules) {
        this.browserslist = browsers;
        this.formatters = userConfig.formatters;
        this.ignoredUrls = ignoredUrls;
        this.parsers = userConfig.parsers;
        this.rules = rules;
        this.extends = userConfig.extends;
        this.rulesTimeout = userConfig.rulesTimeout || 60000;
        if (typeof userConfig.connector === 'string') {
            this.connector = {
                name: userConfig.connector,
                options: {}
            };
        }
        else {
            this.connector = userConfig.connector;
        }
    }
    static loadBrowsersList(config) {
        const directory = process.cwd();
        const files = CONFIG_FILES.reduce((total, configFile) => {
            const filename = path.join(directory, configFile);
            if (shell.test('-f', filename)) {
                total.push(filename);
            }
            return total;
        }, []);
        if (!config.browserslist) {
            for (let i = 0; i < files.length; i++) {
                const file = files[i];
                const tmpConfig = SonarwhalConfig.loadConfigFile(file);
                if (tmpConfig && tmpConfig.browserslist) {
                    config.browserslist = tmpConfig.browserslist;
                    break;
                }
                if (file.endsWith('package.json')) {
                    const packagejson = misc_1.loadJSONFile(file);
                    config.browserslist = packagejson.browserslist;
                }
            }
        }
        if (!config.browserslist || config.browserslist.length === 0) {
            return browserslist();
        }
        return browserslist(config.browserslist);
    }
    static loadConfigFile(filePath) {
        let config;
        switch (path.extname(filePath)) {
            case '':
            case '.json':
                if (path.basename(filePath) === 'package.json') {
                    config = loadPackageJSONConfigFile(filePath);
                }
                else {
                    config = misc_1.loadJSONFile(filePath);
                }
                break;
            case '.js':
                config = misc_1.loadJSFile(filePath);
                break;
            default:
                config = null;
        }
        return config;
    }
    static fromConfig(config, actions) {
        if (!config) {
            throw new Error(`Couldn't find a configuration file`);
        }
        if (!config_validator_1.validateConfig(config)) {
            throw new Error(`Couldn't find any valid configuration`);
        }
        const userConfig = composeConfig(config);
        if (typeof userConfig.connector === 'string') {
            userConfig.connector = {
                name: userConfig.connector,
                options: {}
            };
        }
        if (actions && actions.watch) {
            userConfig.connector.options.watch = actions.watch;
        }
        updateConfigWithCommandLineValues(userConfig, actions);
        if (!Array.isArray(userConfig.formatters)) {
            userConfig.formatters = [userConfig.formatters];
        }
        const browsers = browserslist(config.browserslist);
        const ignoredUrls = loadIgnoredUrls(userConfig);
        const rules = normalize_rules_1.default(userConfig.rules);
        return new SonarwhalConfig(userConfig, browsers, ignoredUrls, normalize_rules_1.default(rules));
    }
    static validateRulesConfig(config) {
        const rules = Object.keys(config.rules);
        const validateResult = rules.reduce((result, rule) => {
            const Rule = resourceLoader.loadRule(rule, config.extends);
            const valid = config_rules_1.validate(Rule.meta, config.rules[rule], rule);
            if (!valid) {
                result.invalid.push(rule);
            }
            else {
                result.valid.push(rule);
            }
            return result;
        }, { invalid: [], valid: [] });
        return validateResult;
    }
    static fromFilePath(filePath, actions) {
        const resolvedPath = path.resolve(process.cwd(), filePath);
        const userConfig = SonarwhalConfig.loadConfigFile(resolvedPath);
        const config = this.fromConfig(userConfig, actions);
        userConfig.browserslist = userConfig.browserslist || SonarwhalConfig.loadBrowsersList(userConfig);
        return config;
    }
}
SonarwhalConfig.getFilenameForDirectory = (directory) => {
    for (let i = 0, len = CONFIG_FILES.length; i < len; i++) {
        const filename = path.join(directory, CONFIG_FILES[i]);
        if (shell.test('-f', filename)) {
            return filename;
        }
    }
    const homedir = os.homedir();
    if (directory === homedir) {
        return null;
    }
    return SonarwhalConfig.getFilenameForDirectory(homedir);
};
exports.SonarwhalConfig = SonarwhalConfig;
