"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chromeLauncher = require("chrome-launcher");
const isCI = require("is-ci");
const lockfile = require("lockfile");
const util_1 = require("util");
const launcher_1 = require("../debugging-protocol-common/launcher");
const logger = require("../../utils/logging");
const debug_1 = require("../../utils/debug");
const misc_1 = require("../../utils/misc");
const debug = debug_1.debug(__filename);
const lock = util_1.promisify(lockfile.lock);
const unlock = util_1.promisify(lockfile.unlock);
class CDPLauncher extends launcher_1.Launcher {
    constructor(options) {
        super(options);
        this.userDataDir = typeof options.defaultProfile === 'boolean' && options.defaultProfile ? false : '';
    }
    async getBrowserInfo() {
        let result = {
            pid: -1,
            port: this.port
        };
        try {
            result = JSON.parse(await misc_1.readFileAsync(this.pidFile));
        }
        catch (e) {
            debug(`Error reading ${this.pidFile}`);
            debug(e);
            result = {
                pid: -1,
                port: this.port
            };
        }
        if (Number.isNaN(result.pid)) {
            return {
                pid: -1,
                port: this.port
            };
        }
        try {
            await misc_1.delay(400);
            process.kill(result.pid, 0);
        }
        catch (e) {
            debug(`Process with ${result.pid} doesn't seem to be running`);
            result = {
                pid: -1,
                port: this.port
            };
        }
        return result;
    }
    async writePid(browserInfo) {
        await misc_1.writeFileAsync(this.pidFile, JSON.stringify({ pid: browserInfo.pid, port: browserInfo.port || this.port }, null, 4));
    }
    async launch(url) {
        const cdpLock = 'cdp.lock';
        try {
            await lock(cdpLock, {
                pollPeriod: 500,
                retries: 20,
                retryWait: 1000,
                stale: 50000,
                wait: 50000
            });
        }
        catch (e) {
            logger.error('Error while locking', e);
            throw e;
        }
        const currentInfo = await this.getBrowserInfo();
        if (currentInfo.pid !== -1) {
            await unlock(cdpLock);
            currentInfo.isNew = false;
            return currentInfo;
        }
        try {
            const chromeFlags = [];
            chromeFlags.push('--no-default-browser-check');
            if (isCI) {
                chromeFlags.push('--headless', '--disable-gpu');
            }
            const chrome = await chromeLauncher.launch({
                chromeFlags,
                logLevel: debug.enabled ? 'verbose' : 'silent',
                startingUrl: url,
                userDataDir: this.userDataDir
            });
            const browserInfo = {
                isNew: true,
                pid: chrome.pid,
                port: chrome.port
            };
            browserInfo.port = browserInfo.port || this.port;
            this.port = browserInfo.port;
            await this.writePid(browserInfo);
            debug('Browser launched correctly');
            await unlock(cdpLock);
            return browserInfo;
        }
        catch (e) {
            debug('Error launching browser');
            debug(e);
            await unlock(cdpLock);
            throw e;
        }
    }
}
exports.CDPLauncher = CDPLauncher;
