"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const cdp = require("chrome-remote-interface");
const _ = require("lodash");
const util_1 = require("util");
const async_html_1 = require("./async-html");
const content_type_1 = require("../../utils/content-type");
const debug_1 = require("../../utils/debug");
const logger = require("../../utils/logging");
const misc_1 = require("../../utils/misc");
const normalize_headers_1 = require("../utils/normalize-headers");
const redirects_1 = require("../utils/redirects");
const requester_1 = require("../utils/requester");
const debug = debug_1.debug(__filename);
class Connector {
    constructor(server, config, launcher) {
        this._errorWithPage = false;
        this._redirects = new redirects_1.RedirectManager();
        this._tabs = [];
        this._faviconLoaded = false;
        const defaultOptions = {
            tabUrl: 'https://empty.sonarwhal.com/',
            useTabUrl: false,
            waitFor: 1000
        };
        this._server = server;
        this._timeout = server.timeout;
        this._options = Object.assign({}, defaultOptions, config);
        this._headers = this._options.headers;
        this._requests = new Map();
        this._pendingResponseReceived = [];
        this.launcher = launcher;
    }
    async getElementFromParser(parts) {
        let basename = null;
        let elements = [];
        while (parts.length > 0) {
            basename = !basename ? parts.pop() : `${parts.pop()}/${basename}`;
            const query = `[src$="${basename}"],[href$="${basename}"]`;
            const newElements = await this._dom.querySelectorAll(query);
            if (newElements.length === 0) {
                if (elements.length > 0) {
                    return elements[0];
                }
                return null;
            }
            if (newElements.length === 1) {
                return newElements[0];
            }
            elements = newElements;
        }
        return elements[0];
    }
    async getElementFromRequest(requestId) {
        const sourceRequest = this._requests.get(requestId);
        if (!sourceRequest) {
            return null;
        }
        const { initiator: { type } } = sourceRequest;
        let { request: { url: requestUrl } } = sourceRequest;
        const originalUrl = this._redirects.calculate(requestUrl);
        requestUrl = new url_1.URL(originalUrl[0] || requestUrl);
        const parts = requestUrl.href.split('/');
        if (['parser', 'other'].includes(type) && requestUrl.protocol.indexOf('http') === 0) {
            return await this.getElementFromParser(parts);
        }
        return null;
    }
    rootFaviconRequestOrResponse(params) {
        if (!this._finalHref) {
            return false;
        }
        const faviconUrl = new url_1.URL('/favicon.ico', this._finalHref).href;
        const event = params.request || params.response;
        if (!event) {
            return false;
        }
        return this._finalHref && faviconUrl === event.url;
    }
    async onRequestWillBeSent(params) {
        const requestUrl = params.request.url;
        this._requests.set(params.requestId, params);
        if (!this._headers) {
            this._headers = params.request.headers;
        }
        if (params.redirectResponse) {
            debug(`Redirect found for ${misc_1.cutString(requestUrl)}`);
            if (requestUrl === params.redirectResponse.url) {
                logger.error(`Error redirecting: ${requestUrl} is an infinite loop`);
                return;
            }
            const hops = this._redirects.calculate(requestUrl);
            if (hops.length >= 10) {
                logger.error(`More than 10 redirects found for ${requestUrl}`);
                return;
            }
            this._redirects.add(requestUrl, params.redirectResponse.url);
            if (hops[0] === this._href) {
                this._finalHref = requestUrl;
            }
            return;
        }
        const eventName = this._href === requestUrl ? 'fetch::start::target' : 'fetch::start';
        debug(`About to start fetching ${misc_1.cutString(requestUrl)}`);
        if (!this.rootFaviconRequestOrResponse(params)) {
            await this._server.emitAsync(eventName, { resource: requestUrl });
        }
    }
    async onLoadingFailed(params) {
        const request = this._requests.get(params.requestId);
        if (!request) {
            debug(`requestId doesn't exist, skipping this error`);
            return;
        }
        const requestUrl = request.request.url;
        if (requestUrl === this._href || requestUrl === this._finalHref) {
            this._errorWithPage = true;
            return;
        }
        if (!this._dom) {
            this._pendingResponseReceived.push(this.onLoadingFailed.bind(this, params));
            return;
        }
        debug(`Error found:\n${JSON.stringify(params)}`);
        const element = await this.getElementFromRequest(params.requestId);
        const requestInfo = this._requests.get(params.requestId);
        if (!requestInfo) {
            debug(`Request ${params.requestId} failed but wasn't in the list`);
            return;
        }
        const { request: { url: resource } } = requestInfo;
        const eventName = 'fetch::error';
        const hops = this._redirects.calculate(resource);
        const event = {
            element,
            error: params,
            hops,
            resource
        };
        this._requests.delete(params.requestId);
        if (!this.rootFaviconRequestOrResponse(params)) {
            await this._server.emitAsync(eventName, event);
        }
    }
    async getResponseBody(cdpResponse) {
        let content = '';
        let rawContent = null;
        const rawResponse = () => {
            return Promise.resolve(null);
        };
        const fetchContent = this.fetchContent.bind(this);
        const defaultBody = { content, rawContent, rawResponse };
        if (cdpResponse.response.status !== 200) {
            return defaultBody;
        }
        try {
            const { body, base64Encoded } = await this._client.Network.getResponseBody({ requestId: cdpResponse.requestId });
            const encoding = base64Encoded ? 'base64' : 'utf8';
            content = body;
            rawContent = Buffer.from(body, encoding);
            const returnValue = {
                content,
                rawContent,
                rawResponse() {
                    const self = this;
                    const cached = self._rawResponse;
                    if (cached) {
                        return Promise.resolve(cached);
                    }
                    if (rawContent.length.toString() === cdpResponse.response.headers['Content-Length']) {
                        return Promise.resolve(rawContent);
                    }
                    const { url: responseUrl, requestHeaders: headers } = cdpResponse.response;
                    const validHeaders = Object.entries(headers || {}).reduce((final, [key, value]) => {
                        if (key.startsWith(':')) {
                            return final;
                        }
                        final[key] = value;
                        return final;
                    }, {});
                    return fetchContent(responseUrl, validHeaders)
                        .then((result) => {
                        const { response: { body: { rawResponse: rr } } } = result;
                        return rr();
                    })
                        .then((value) => {
                        self._rawResponse = value;
                        return value;
                    });
                }
            };
            debug(`Content for ${misc_1.cutString(cdpResponse.response.url)} downloaded`);
            return returnValue;
        }
        catch (e) {
            debug(`Body requested after connection closed for request ${cdpResponse.requestId}`);
            defaultBody.rawContent = Buffer.alloc(0);
            debug(`Content for ${misc_1.cutString(cdpResponse.response.url)} downloaded`);
            return defaultBody;
        }
    }
    async createResponse(cdpResponse, element) {
        const resourceUrl = cdpResponse.response.url;
        const hops = this._redirects.calculate(resourceUrl);
        const resourceHeaders = normalize_headers_1.normalizeHeaders(cdpResponse.response.headers);
        const { content, rawContent, rawResponse } = await this.getResponseBody(cdpResponse);
        const response = {
            body: {
                content,
                rawContent,
                rawResponse
            },
            charset: null,
            headers: resourceHeaders,
            hops,
            mediaType: null,
            statusCode: cdpResponse.response.status,
            url: resourceUrl
        };
        const { charset, mediaType } = content_type_1.getContentTypeData(element, resourceUrl, response.headers, response.body.rawContent);
        response.mediaType = mediaType;
        response.charset = charset;
        return response;
    }
    async onResponseReceived(params) {
        const resourceUrl = params.response.url;
        const hops = this._redirects.calculate(resourceUrl);
        const originalUrl = hops[0] || resourceUrl;
        let element = null;
        let eventName = this._href === originalUrl ? 'fetch::end::html' : 'fetch::end';
        if (eventName !== 'fetch::end::html') {
            if (!this._dom) {
                this._pendingResponseReceived.push(this.onResponseReceived.bind(this, params));
                return;
            }
            try {
                element = await this.getElementFromRequest(params.requestId);
            }
            catch (e) {
                debug(`Error finding element for request ${params.requestId}. element will be null`);
            }
        }
        const response = await this.createResponse(params, element);
        const request = {
            headers: params.response.requestHeaders,
            url: originalUrl
        };
        const data = {
            element,
            request,
            resource: resourceUrl,
            response
        };
        if (eventName === 'fetch::end::html') {
            this._targetNetworkData = {
                request,
                response
            };
        }
        else if (eventName === 'fetch::end') {
            eventName = `${eventName}::${content_type_1.getType(response.mediaType)}`;
        }
        if (misc_1.hasAttributeWithValue(data.element, 'link', 'rel', 'icon')) {
            this._faviconLoaded = true;
        }
        if (!this.rootFaviconRequestOrResponse(params)) {
            await this._server.emitAsync(eventName, data);
        }
        this._requests.delete(params.requestId);
    }
    async traverseAndNotify(element) {
        const ignoredNodeTypes = [10];
        if (ignoredNodeTypes.includes(element.nodeType)) {
            return;
        }
        const eventName = `element::${element.nodeName.toLowerCase()}`;
        const wrappedElement = new async_html_1.AsyncHTMLElement(element, this._dom, this._client.DOM);
        debug(`emitting ${eventName}`);
        const event = {
            element: wrappedElement,
            resource: this._finalHref
        };
        await this._server.emitAsync(eventName, event);
        const elementChildren = wrappedElement.children;
        for (const child of elementChildren) {
            debug('next children');
            const traverseDown = {
                element,
                resource: this._finalHref
            };
            await this._server.emitAsync(`traverse::down`, traverseDown);
            await this.traverseAndNotify(child);
        }
        const traverseUp = {
            element,
            resource: this._finalHref
        };
        await this._server.emitAsync(`traverse::up`, traverseUp);
    }
    getClient(port, tab) {
        let retries = 0;
        const loadCDP = async () => {
            try {
                const client = await cdp({ port, tab });
                return client;
            }
            catch (err) {
                if (retries > 3) {
                    throw err;
                }
                await misc_1.delay((retries * 250) + 500);
                retries++;
                return loadCDP();
            }
        };
        return loadCDP();
    }
    async initiateComms() {
        const launcher = await this.launcher.launch(this._options.useTabUrl ? this._options.tabUrl : 'about:blank');
        let client;
        this.pid = launcher.pid;
        if (launcher.isNew) {
            const tabs = _.filter(await cdp.List({ port: launcher.port }), (tab) => {
                return !tab.url.startsWith('chrome-extension');
            });
            client = await this.getClient(launcher.port, tabs[0]);
            this._tabs = tabs;
        }
        else {
            const tab = await cdp.New({ port: launcher.port, url: this._options.useTabUrl ? this._options.tabUrl : null });
            if (!tab) {
                throw new Error('Error trying to open a new tab');
            }
            this._tabs.push(tab);
            client = await cdp({
                port: launcher.port,
                tab: (tabs) => {
                    for (let index = 0; index < tabs.length; index++) {
                        if (tabs[index].id === tab.id) {
                            return index;
                        }
                    }
                    return -1;
                }
            });
        }
        return client;
    }
    onError(err) {
        debug(`Error: \n${err}`);
    }
    onDisconnect() {
        debug(`Disconnected`);
    }
    async enableNetworkEvents() {
        debug('Binding to Network events');
        const { Network } = this._client;
        await Promise.all([
            Network.clearBrowserCache(),
            Network.setCacheDisabled({ cacheDisabled: true }),
            Network.requestWillBeSent(this.onRequestWillBeSent.bind(this)),
            Network.responseReceived(this.onResponseReceived.bind(this)),
            Network.loadingFailed(this.onLoadingFailed.bind(this))
        ]);
    }
    async configureAndEnableCDP() {
        const { Network, Page } = this._client;
        this._client.on('error', this.onError);
        this._client.on('disconnect', this.onDisconnect);
        await this.enableNetworkEvents();
        await Promise.all([
            Network.enable(),
            Page.enable()
        ]);
    }
    async getFavicon(element) {
        const href = (element && element.getAttribute('href')) || '/favicon.ico';
        try {
            debug(`resource ${href} to be fetched`);
            const fullFaviconUrl = this._finalHref + href.substr(1);
            await this._server.emitAsync('fetch::start', { resource: fullFaviconUrl });
            const content = await this.fetchContent(new url_1.URL(fullFaviconUrl));
            const data = {
                element: null,
                request: content.request,
                resource: content.response.url,
                response: content.response
            };
            await this._server.emitAsync('fetch::end::image', data);
        }
        catch (error) {
            const hops = this._redirects.calculate(href);
            const event = {
                element,
                error,
                hops,
                resource: href
            };
            await this._server.emitAsync('fetch::error', event);
        }
    }
    async processPendingResponses() {
        while (this._pendingResponseReceived.length) {
            debug(`Pending requests: ${this._pendingResponseReceived.length}`);
            await this._pendingResponseReceived.shift()();
        }
    }
    onLoadEventFired(callback) {
        return async () => {
            await misc_1.delay(this._options.waitFor);
            const { DOM } = this._client;
            const event = { resource: this._finalHref };
            try {
                this._dom = new async_html_1.AsyncHTMLDocument(DOM);
                await this._dom.load();
                await this.processPendingResponses();
                if (this._errorWithPage) {
                    return callback(new Error('Problem loading the website'));
                }
                await this._server.emitAsync('traverse::start', event);
                await this.traverseAndNotify(this._dom.root);
                await this._server.emitAsync('traverse::end', event);
                if (!this._faviconLoaded) {
                    const faviconElement = (await this._dom.querySelectorAll('link[rel~="icon"]'))[0];
                    await this.getFavicon(faviconElement);
                }
                return setTimeout(async () => {
                    await this._server.emitAsync('scan::end', event);
                    return callback();
                }, 1000);
            }
            catch (err) {
                return callback(err);
            }
        };
    }
    collect(target) {
        if (!target.protocol.match(/https?:/)) {
            const err = {
                message: `Protocol "${target.protocol}" invalid for the current collector`,
                type: 'InvalidTarget'
            };
            throw err;
        }
        return util_1.promisify(async (callback) => {
            this._href = target.href.replace(target.hash, '');
            this._finalHref = target.href;
            const event = { resource: target.href };
            let client;
            await this._server.emit('scan::start', event);
            try {
                client = await this.initiateComms();
            }
            catch (e) {
                debug('Error connecting to browser');
                debug(e);
                callback(e);
                return;
            }
            this._client = client;
            const { Page, Security } = client;
            if (this._options.overrideInvalidCert) {
                Security.certificateError(({ eventId }) => {
                    Security.handleCertificateError({
                        action: 'continue',
                        eventId
                    });
                });
                await Security.enable();
                await Security.setOverrideCertificateErrors({ override: true });
            }
            const loadHandler = this.onLoadEventFired(callback);
            Page.loadEventFired(loadHandler);
            try {
                await this.configureAndEnableCDP();
                await Page.navigate({ url: target.href });
            }
            catch (e) {
                await this._server.emitAsync('scan::end', event);
                callback(e);
                return;
            }
        })();
    }
    isClosed() {
        return new Promise(async (resolve) => {
            let maxTries = 200;
            let finish = false;
            while (!finish) {
                try {
                    process.kill(this.pid, 0);
                    maxTries--;
                    if (maxTries === 0) {
                        finish = true;
                    }
                    else {
                        await misc_1.delay(50);
                    }
                }
                catch (e) {
                    debug(`Process with ${this.pid} doesn't seem to be running`);
                    finish = true;
                }
            }
            resolve();
        });
    }
    async close() {
        debug('Closing browsers used by CDP');
        while (this._tabs.length > 0) {
            const tab = this._tabs.pop();
            try {
                await cdp.closeTab({ id: tab.id, port: this._client.port });
            }
            catch (e) {
                debug(`Couldn't close tab ${tab.id}`);
            }
        }
        try {
            this._client.close();
            await this.isClosed();
        }
        catch (e) {
            debug(`Couldn't close the client properly`);
        }
    }
    async fetchContent(target, customHeaders) {
        const assigns = _.compact([this && this._headers, customHeaders]);
        const headers = Object.assign({}, ...assigns);
        const href = typeof target === 'string' ? target : target.href;
        const options = {
            headers,
            rejectUnauthorized: !this._options.overrideInvalidCert,
            strictSSL: !this._options.overrideInvalidCert
        };
        const request = new requester_1.Requester(options);
        const response = await request.get(href);
        return response;
    }
    wrapRuntimeEvalErrorInBrowser(e) {
        const err = e || new Error();
        const fallbackMessage = typeof err === 'string' ? err : 'unknown error';
        return {
            __failedInBrowser: true,
            message: err.message || fallbackMessage,
            name: err.name || 'Error',
            stack: err.stack || (new Error()).stack
        };
    }
    evaluate(code) {
        return new Promise(async (resolve, reject) => {
            const asyncTimeout = setTimeout((() => {
                reject(new Error('The asynchronous expression exceeded the allotted time of 60s'));
            }), this._timeout);
            try {
                const expression = `(function wrapInNativePromise() {
          const __nativePromise = window.__nativePromise || Promise;
          return new __nativePromise(function (resolve) {
            return __nativePromise.resolve()
              .then(_ => ${code})
              .catch(function ${this.wrapRuntimeEvalErrorInBrowser.toString()})
              .then(resolve);
          });
        }())`;
                const result = await this._client.Runtime.evaluate({
                    awaitPromise: true,
                    expression,
                    includeCommandLineAPI: true,
                    returnByValue: true
                });
                clearTimeout(asyncTimeout);
                const value = result.result.value;
                if (result.exceptionDetails) {
                    return reject(new Error('an unexpected driver error occurred'));
                }
                if (value && value.__failedInBrowser) {
                    return reject(Object.assign(new Error(), value));
                }
                return resolve(value);
            }
            catch (err) {
                clearTimeout(asyncTimeout);
                return reject(err);
            }
        });
    }
    querySelectorAll(selector) {
        return this._dom.querySelectorAll(selector);
    }
    get dom() {
        return this._dom;
    }
    get headers() {
        return this._targetNetworkData.response.headers;
    }
    get html() {
        return this._dom.pageHTML();
    }
}
exports.Connector = Connector;
