"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const url = require("url");
const url_1 = require("url");
const util = require("util");
const child_process_1 = require("child_process");
const jsdom = require("jsdom/lib/old-api");
const debug_1 = require("../../utils/debug");
const content_type_1 = require("../../utils/content-type");
const jsdom_async_html_1 = require("./jsdom-async-html");
const requester_1 = require("../utils/requester");
const debug = debug_1.debug(__filename);
const defaultOptions = { waitFor: 1000 };
class JSDOMConnector {
    constructor(server, config) {
        this.killProcess = (runner) => {
            try {
                runner.kill('SIGKILL');
            }
            catch (err) {
                debug('Error closing evaluate process');
            }
        };
        this._options = Object.assign({}, defaultOptions, config);
        this._headers = this._options.headers;
        this._request = new requester_1.Requester(this._options);
        this._server = server;
        this._timeout = server.timeout;
    }
    _fetchUrl(target, customHeaders) {
        const uri = url.format(target);
        if (!customHeaders) {
            return this._request.get(uri);
        }
        const r = new requester_1.Requester({
            headers: customHeaders,
            rejectUnauthorized: this._options.rejectUnauthorized,
            strictSSL: this._options.strictSSL
        });
        return r.get(uri);
    }
    async traverseAndNotify(element) {
        const eventName = `element::${element.nodeName.toLowerCase()}`;
        debug(`emitting ${eventName}`);
        const event = {
            element: new jsdom_async_html_1.JSDOMAsyncHTMLElement(element),
            resource: this._finalHref
        };
        await this._server.emitAsync(eventName, event);
        for (let i = 0; i < element.children.length; i++) {
            const child = element.children[i];
            debug('next children');
            const traverseDown = {
                element: new jsdom_async_html_1.JSDOMAsyncHTMLElement(element),
                resource: this._finalHref
            };
            await this._server.emitAsync(`traverse::down`, traverseDown);
            await this.traverseAndNotify(child);
        }
        const traverseUp = {
            element: new jsdom_async_html_1.JSDOMAsyncHTMLElement(element),
            resource: this._finalHref
        };
        await this._server.emitAsync(`traverse::up`, traverseUp);
        return Promise.resolve();
    }
    async resourceLoader(resource, callback) {
        let resourceUrl = resource.url.href;
        const element = resource.element ? new jsdom_async_html_1.JSDOMAsyncHTMLElement(resource.element) : null;
        if (!resource.url.protocol) {
            resourceUrl = new url_1.URL(resource.url.href, this._finalHref).href;
        }
        if (this._fetchedHrefs.has(resourceUrl)) {
            return callback(null, '');
        }
        this._fetchedHrefs.add(resourceUrl);
        debug(`resource ${resourceUrl} to be fetched`);
        await this._server.emitAsync('fetch::start', { resource: resourceUrl });
        try {
            const resourceNetworkData = await this.fetchContent(resourceUrl);
            debug(`resource ${resourceUrl} fetched`);
            const fetchEndEvent = {
                element,
                request: resourceNetworkData.request,
                resource: resourceNetworkData.response.url,
                response: resourceNetworkData.response
            };
            const { charset, mediaType } = content_type_1.getContentTypeData(element, fetchEndEvent.resource, fetchEndEvent.response.headers, fetchEndEvent.response.body.rawContent);
            const type = content_type_1.getType(mediaType);
            fetchEndEvent.response.mediaType = mediaType;
            fetchEndEvent.response.charset = charset;
            await this._server.emitAsync(`fetch::end::${type}`, fetchEndEvent);
            return callback(null, resourceNetworkData.response.body.content);
        }
        catch (err) {
            const hops = this._request.getRedirects(err.uri);
            const fetchError = {
                element,
                error: err.error,
                hops,
                resource: err.uri || resourceUrl
            };
            await this._server.emitAsync('fetch::error', fetchError);
            return callback(fetchError);
        }
    }
    async getFavicon(element) {
        const href = (element && element.getAttribute('href')) || '/favicon.ico';
        try {
            await util.promisify(this.resourceLoader).call(this, { element, url: new url_1.URL(href, this._finalHref) });
        }
        catch (e) {
            debug('Error loading ${href}', e);
        }
    }
    collect(target) {
        if (!target.protocol.match(/https?:/)) {
            const err = {
                message: `Protocol "${target.protocol}" is invalid for the current collector`,
                type: 'InvalidTarget'
            };
            throw err;
        }
        const href = this._href = target.href;
        const initialEvent = { resource: href };
        this._fetchedHrefs = new Set();
        this._server.emit('scan::start', initialEvent);
        return new Promise(async (resolve, reject) => {
            debug(`About to start fetching ${href}`);
            await this._server.emitAsync('fetch::start::target', initialEvent);
            try {
                this._targetNetworkData = await this.fetchContent(target);
            }
            catch (err) {
                const hops = this._request.getRedirects(err.uri);
                const fetchError = {
                    element: null,
                    error: err.error ? err.error : err,
                    hops,
                    resource: href
                };
                await this._server.emitAsync('fetch::error', fetchError);
                debug(`Failed to fetch: ${href}\n${err}`);
                await this._server.emitAsync('scan::end', initialEvent);
                reject(fetchError);
                return;
            }
            this._finalHref = this._targetNetworkData.response.url;
            debug(`HTML for ${this._finalHref} downloaded`);
            const fetchEnd = {
                element: null,
                request: this._targetNetworkData.request,
                resource: this._finalHref,
                response: this._targetNetworkData.response
            };
            const { charset, mediaType } = content_type_1.getContentTypeData(fetchEnd.element, fetchEnd.resource, fetchEnd.response.headers, fetchEnd.response.body.rawContent);
            fetchEnd.response.mediaType = mediaType;
            fetchEnd.response.charset = charset;
            await this._server.emitAsync('fetch::end::html', fetchEnd);
            jsdom.env({
                done: async (err, window) => {
                    if (err) {
                        await this._server.emitAsync('scan::end', { resource: this._finalHref });
                        reject(err);
                        return;
                    }
                    this._window = window;
                    this._document = new jsdom_async_html_1.JSDOMAsyncHTMLDocument(window.document);
                    setTimeout(async () => {
                        const event = { resource: this._finalHref };
                        debug(`${this._finalHref} loaded, traversing`);
                        try {
                            await this._server.emitAsync('traverse::start', event);
                            await this.traverseAndNotify(window.document.children[0]);
                            await this._server.emitAsync('traverse::end', event);
                            await this.getFavicon(window.document.querySelector('link[rel~="icon"]'));
                            await this._server.emitAsync('scan::end', event);
                        }
                        catch (e) {
                            reject(e);
                        }
                        resolve();
                    }, this._options.waitFor);
                },
                features: {
                    FetchExternalResources: ['script', 'link', 'img'],
                    ProcessExternalResources: ['script'],
                    SkipExternalResources: false
                },
                headers: this._headers,
                html: this._targetNetworkData.response.body.content,
                resourceLoader: this.resourceLoader.bind(this),
                url: this._finalHref
            });
        });
    }
    close() {
        try {
            this._window.close();
        }
        catch (e) {
            debug(`Exception ignored while closing JSDOM connector (most likely pending network requests)`);
            debug(e);
        }
        return Promise.resolve();
    }
    fetchContent(target, customHeaders) {
        let parsedTarget = target;
        if (typeof parsedTarget === 'string') {
            parsedTarget = parsedTarget.indexOf('//') === 0 ? `http:${parsedTarget}` : parsedTarget;
            parsedTarget = new url_1.URL(parsedTarget);
            return this.fetchContent(parsedTarget, customHeaders);
        }
        return this._fetchUrl(parsedTarget, customHeaders);
    }
    evaluate(source) {
        return new Promise((resolve, reject) => {
            const runner = child_process_1.fork(path.join(__dirname, 'evaluate-runner'), [this._finalHref || this._href, this._options.waitFor], { execArgv: [] });
            let timeoutId;
            runner.on('message', (result) => {
                if (timeoutId) {
                    clearTimeout(timeoutId);
                    timeoutId = null;
                }
                this.killProcess(runner);
                if (result.error) {
                    return reject(result.error);
                }
                return resolve(result.evaluate);
            });
            runner.send({ source });
            timeoutId = setTimeout(() => {
                debug(`Evaluation timed out after ${this._timeout / 1000}s. Killing process and reporting an error.`);
                this.killProcess(runner);
                return reject(new Error('TIMEOUT'));
            }, this._timeout);
        });
    }
    querySelectorAll(selector) {
        return this._document.querySelectorAll(selector);
    }
    get dom() {
        return this._document;
    }
    get headers() {
        return this._targetNetworkData.response.headers;
    }
    get html() {
        return this._document.pageHTML();
    }
}
exports.default = JSDOMConnector;
