"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url = require("url");
const path = require("path");
const chokidar = require("chokidar");
const globby = require("globby");
const fs = require("fs-extra");
const debug_1 = require("../../utils/debug");
const get_as_uri_1 = require("../../utils/get-as-uri");
const get_as_path_string_1 = require("../../utils/get-as-path-string");
const content_type_1 = require("../../utils/content-type");
const misc_1 = require("../../utils/misc");
const logger = require("../../utils/logging");
const debug = debug_1.debug(__filename);
const defaultOptions = {};
class LocalConnector {
    constructor(sonarwhal, config) {
        this.getGitIgnore = async () => {
            try {
                const rawList = await misc_1.readFileAsync(path.join(process.cwd(), '.gitignore'));
                const splitList = rawList.split('\n');
                const result = splitList.reduce((total, ignore) => {
                    const value = ignore.trim();
                    if (!value) {
                        return total;
                    }
                    if (value[0] === '/') {
                        total.push(value.substr(1));
                    }
                    else {
                        total.push(value);
                    }
                    return total;
                }, []);
                return result;
            }
            catch (err) {
                logger.error('Error reading .gitignore');
                return [];
            }
        };
        this._options = Object.assign({}, defaultOptions, config);
        this.filesPattern = this.getFilesPattern();
        this.sonarwhal = sonarwhal;
    }
    getFilesPattern() {
        const pattern = this._options.pattern;
        if (!pattern) {
            return ['**', '!.git/**'];
        }
        if (Array.isArray(pattern)) {
            return pattern.length > 0 ? pattern : null;
        }
        return [pattern];
    }
    async fetch(filePath) {
        const event = await this.fetchContent(filePath);
        const type = content_type_1.getType(event.response.mediaType);
        await this.sonarwhal.emitAsync(`fetch::end::${type}`, event);
    }
    async notify() {
        const href = this._href;
        const scanEndEvent = { resource: href };
        await this.sonarwhal.emitAsync('scan::end', scanEndEvent);
        await this.sonarwhal.notify();
        logger.log('Watching for file changes.');
    }
    watch(targetString) {
        return new Promise(async (resolve, reject) => {
            const isF = misc_1.isFile(targetString);
            const target = isF ? targetString : '.';
            const ignored = await this.getGitIgnore();
            this.watcher = chokidar.watch(target, {
                cwd: !isF ? targetString : null,
                ignored: ignored.concat(['.git/']),
                ignoreInitial: true,
                ignorePermissionErrors: true
            });
            const getFile = (filePath) => {
                if (isF) {
                    return filePath;
                }
                if (path.isAbsolute(filePath)) {
                    return filePath;
                }
                return path.join(targetString, filePath);
            };
            const onAdd = async (filePath) => {
                const file = getFile(filePath);
                logger.log(`File ${file} added`);
                await this.fetch(file);
                await this.notify();
            };
            const onChange = async (filePath) => {
                const file = getFile(filePath);
                const fileUrl = get_as_uri_1.getAsUri(file);
                logger.log(`File ${file} changeg`);
                this.sonarwhal.clean(fileUrl);
                await this.fetch(file);
                await this.notify();
            };
            const onUnlink = async (filePath) => {
                const file = getFile(filePath);
                const fileUrl = get_as_uri_1.getAsUri(file);
                this.sonarwhal.clean(fileUrl);
                logger.log('onUnlink');
                await this.notify();
            };
            const onReady = async () => {
                await this.notify();
            };
            const onError = (err) => {
                logger.error('error', err);
                reject(err);
            };
            this.watcher
                .on('add', onAdd.bind(this))
                .on('change', onChange.bind(this))
                .on('unlink', onUnlink.bind(this))
                .on('error', onError)
                .on('ready', onReady);
            process.once('SIGINT', () => {
                this.watcher.close();
                this.sonarwhal.clear();
                resolve();
            });
        });
    }
    async fetchContent(target) {
        const uri = get_as_uri_1.getAsUri(target);
        const filePath = get_as_path_string_1.getAsPathString(uri);
        const rawContent = await fs.readFile(filePath);
        const contentType = content_type_1.getContentTypeData(null, filePath, null, rawContent);
        let content = '';
        if (content_type_1.isTextMediaType(contentType.mediaType)) {
            content = rawContent.toString(contentType.charset);
        }
        return {
            element: null,
            request: null,
            resource: url.format(get_as_uri_1.getAsUri(filePath)),
            response: {
                body: {
                    content,
                    rawContent,
                    rawResponse() {
                        return Promise.resolve(rawContent);
                    }
                },
                charset: contentType.charset,
                headers: null,
                hops: [],
                mediaType: contentType.mediaType,
                statusCode: 200,
                url: filePath
            }
        };
    }
    async collect(target) {
        const href = this._href = target.href;
        const initialEvent = { resource: href };
        this.sonarwhal.emitAsync('scan::start', initialEvent);
        const pathString = get_as_path_string_1.getAsPathString(target);
        let files;
        if (misc_1.isFile(pathString)) {
            await this.sonarwhal.emitAsync('fetch::start::target', initialEvent);
            files = [pathString];
        }
        else {
            files = await globby(this.filesPattern, {
                absolute: true,
                cwd: pathString,
                dot: true,
                gitignore: true
            });
        }
        await Promise.all(files.map(this.fetch.bind(this)));
        if (this._options.watch) {
            await this.watch(pathString);
        }
        else {
            await this.sonarwhal.emitAsync('scan::end', initialEvent);
        }
    }
    close() {
        return Promise.resolve();
    }
}
exports.default = LocalConnector;
