"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url = require("url");
const util_1 = require("util");
const zlib = require("zlib");
const async = require("async");
const brotli = require("iltorb");
const request = require("request");
const iconv = require("iconv-lite");
const debug_1 = require("../../utils/debug");
const misc_1 = require("../../utils/misc");
const content_type_1 = require("../../utils/content-type");
const redirects_1 = require("./redirects");
const debug = debug_1.debug(__filename);
const decompressBrotli = util_1.promisify(brotli.decompress);
const decompressGzip = util_1.promisify(zlib.gunzip);
const inflateAsync = util_1.promisify(zlib.inflate);
const inflateRawAsync = util_1.promisify(zlib.inflateRaw);
const inflate = (buff) => {
    if ((buff[0] & 0x0f) === 8 && (buff.readUInt16BE(0) % 31 === 0)) {
        return inflateAsync(buff);
    }
    return inflateRawAsync(buff);
};
const identity = (buff) => {
    return Promise.resolve(Buffer.from(buff));
};
const asyncSome = util_1.promisify(async.someSeries);
const defaults = {
    encoding: null,
    followRedirect: false,
    headers: {
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'en-US,en;q=0.8,es;q=0.6,fr;q=0.4',
        'Cache-Control': 'no-cache',
        DNT: 1,
        Pragma: 'no-cache',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.2924.87 Safari/537.36'
    },
    jar: true,
    time: true,
    timeout: 10000
};
class Requester {
    constructor(customOptions) {
        this._redirects = new redirects_1.RedirectManager();
        this._maxRedirects = 10;
        if (customOptions) {
            customOptions.followRedirect = false;
            customOptions.rejectUnauthorized = false;
            this._maxRedirects = customOptions.maxRedirects || this._maxRedirects;
            if (customOptions.headers) {
                customOptions.headers = Object.assign({}, misc_1.toLowerCaseKeys(defaults.headers), misc_1.toLowerCaseKeys(customOptions.headers));
            }
        }
        const options = Object.assign({}, defaults, customOptions);
        this._request = request.defaults(options);
    }
    async tryToDecompress(decompressor, content) {
        try {
            const result = await decompressor(content);
            return result;
        }
        catch (e) {
            return null;
        }
    }
    decompressors(algorithm) {
        const priorities = {
            br: 0,
            gzip: 1,
            deflate: 2,
            identity: 3
        };
        const functions = [
            decompressBrotli,
            decompressGzip,
            inflate,
            identity
        ];
        const priority = typeof priorities[algorithm.trim()] === 'undefined' ?
            priorities.identity :
            priorities[algorithm];
        return functions.slice(priority);
    }
    async decompressResponse(contentEncoding, rawBodyResponse) {
        const that = this;
        const algorithms = contentEncoding ?
            contentEncoding.split(',') :
            [''];
        const decompressors = this.decompressors(algorithms.shift().trim());
        let rawBody;
        await asyncSome(decompressors, async (decompressor) => {
            rawBody = await that.tryToDecompress(decompressor, rawBodyResponse);
            return !!rawBody;
        });
        if (algorithms.length > 0) {
            return this.decompressResponse(algorithms.join(','), rawBody);
        }
        return rawBody;
    }
    getRedirects(uri) {
        return this._redirects.calculate(uri);
    }
    get(uri) {
        debug(`Requesting ${uri}`);
        return new Promise((resolve, reject) => {
            const byteChunks = [];
            let rawBodyResponse;
            this._request({ uri }, async (err, response) => {
                if (err) {
                    debug(`Request for ${uri} failed\n${err}`);
                    return reject({
                        error: err,
                        uri
                    });
                }
                if (Requester.validRedirects.includes(response.statusCode)) {
                    const newUri = url.resolve(uri, response.headers.location);
                    this._redirects.add(newUri, uri);
                    const currentRedirectNumber = this._redirects.calculate(newUri).length;
                    if (currentRedirectNumber > this._maxRedirects) {
                        return reject(`The number of redirects(${currentRedirectNumber}) exceeds the limit(${this._maxRedirects}).`);
                    }
                    try {
                        debug(`Redirect found for ${uri}`);
                        const results = await this.get(newUri);
                        return resolve(results);
                    }
                    catch (e) {
                        return reject(e);
                    }
                }
                const contentEncoding = misc_1.getHeaderValueNormalized(response.headers, 'content-encoding');
                const rawBody = await this.decompressResponse(contentEncoding, rawBodyResponse);
                const { charset, mediaType } = content_type_1.getContentTypeData(null, uri, response.headers, rawBody);
                const hops = this._redirects.calculate(uri);
                const body = iconv.encodingExists(charset) ? iconv.decode(rawBody, charset) : null;
                const networkData = {
                    request: {
                        headers: response.request.headers,
                        url: hops[0] || uri
                    },
                    response: {
                        body: {
                            content: body,
                            rawContent: rawBody,
                            rawResponse: () => {
                                return Promise.resolve(rawBodyResponse);
                            }
                        },
                        charset,
                        headers: response.headers,
                        hops,
                        mediaType,
                        statusCode: response.statusCode,
                        url: uri
                    }
                };
                return resolve(networkData);
            })
                .on('response', (response) => {
                response
                    .on('data', (data) => {
                    byteChunks.push(data);
                })
                    .on('end', () => {
                    rawBodyResponse = Buffer.concat(byteChunks);
                });
            });
        });
    }
}
Requester.validRedirects = [301, 302, 303, 307, 308];
exports.Requester = Requester;
