"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url = require("url");
const chalk_1 = require("chalk");
const eventemitter2_1 = require("eventemitter2");
const _ = require("lodash");
const debug_1 = require("./utils/debug");
const config_rules_1 = require("./config/config-rules");
const logger = require("./utils/logging");
const rule_context_1 = require("./rule-context");
const rulescope_1 = require("./enums/rulescope");
const debug = debug_1.debug(__filename);
class Sonarwhal extends eventemitter2_1.EventEmitter2 {
    constructor(config, resources) {
        super({
            delimiter: '::',
            maxListeners: 0,
            wildcard: true
        });
        this.browserslist = [];
        this._timeout = 60000;
        debug('Initializing sonarwhal engine');
        this._timeout = config.rulesTimeout;
        this.messages = [];
        this.browserslist = config.browserslist;
        this.ignoredUrls = config.ignoredUrls;
        const Connector = resources.connector;
        const connectorId = config.connector.name;
        if (!Connector) {
            throw new Error(`Connector "${connectorId}" not found`);
        }
        this.connector = new Connector(this, config.connector.options);
        this._formatters = resources.formatters.map((Formatter) => {
            return new Formatter();
        });
        this.parsers = resources.parsers.map((Parser) => {
            debug(`Loading parser`);
            return new Parser(this);
        });
        this.rules = new Map();
        resources.rules.forEach((Rule) => {
            debug('Loading rules');
            const id = Rule.meta.id;
            const ignoreRule = (RuleCtor) => {
                const ignoredConnectors = RuleCtor.meta.ignoredConnectors || [];
                return (connectorId === 'local' && RuleCtor.meta.scope === rulescope_1.RuleScope.site) ||
                    (connectorId !== 'local' && RuleCtor.meta.scope === rulescope_1.RuleScope.local) ||
                    ignoredConnectors.includes(connectorId);
            };
            const ruleOptions = config.rules[id];
            const severity = config_rules_1.getSeverity(ruleOptions);
            if (ignoreRule(Rule)) {
                debug(`Rule "${id}" is disabled for the connector "${connectorId}"`);
                logger.log(chalk_1.default.yellow(`Warning: The rule "${id}" will be ignored for the connector "${connectorId}"`));
            }
            else if (severity) {
                const context = new rule_context_1.RuleContext(id, this, severity, ruleOptions, Rule.meta);
                const rule = new Rule(context);
                this.rules.set(id, rule);
            }
            else {
                debug(`Rule "${id}" is disabled`);
            }
        });
    }
    get pageDOM() {
        return this.connector.dom;
    }
    get pageContent() {
        return this.connector.html;
    }
    get pageHeaders() {
        return this.connector.headers;
    }
    get targetedBrowsers() {
        return this.browserslist;
    }
    get formatters() {
        return this._formatters;
    }
    get timeout() {
        return this._timeout;
    }
    isIgnored(urls, resource) {
        if (!urls) {
            return false;
        }
        return urls.some((urlIgnored) => {
            return urlIgnored.test(resource);
        });
    }
    onRuleEvent(id, eventName, listener) {
        const that = this;
        const createEventHandler = (handler, ruleId) => {
            return function (event) {
                const urlsIgnoredForAll = that.ignoredUrls.get('all') ? that.ignoredUrls.get('all') : [];
                const urlsIgnoredForRule = that.ignoredUrls.get(ruleId) ? that.ignoredUrls.get(ruleId) : [];
                const urlsIgnored = urlsIgnoredForRule.concat(urlsIgnoredForAll);
                if (that.isIgnored(urlsIgnored, event.resource)) {
                    return null;
                }
                return new Promise((resolve) => {
                    let immediateId;
                    const timeoutId = setTimeout(() => {
                        if (immediateId) {
                            clearImmediate(immediateId);
                            immediateId = null;
                        }
                        debug(`Rule ${ruleId} timeout`);
                        resolve(null);
                    }, that._timeout);
                    immediateId = setImmediate(async () => {
                        const result = await handler(event, this.event);
                        if (timeoutId) {
                            clearTimeout(timeoutId);
                        }
                        resolve(result);
                    });
                });
            };
        };
        this.on(eventName, createEventHandler(listener, id));
    }
    fetchContent(target, headers) {
        return this.connector.fetchContent(target, headers);
    }
    evaluate(source) {
        return this.connector.evaluate(source);
    }
    async close() {
        await this.connector.close();
    }
    report(ruleId, severity, sourceCode, location, message, resource) {
        const problem = {
            location: location || { column: -1, line: -1 },
            message,
            resource,
            ruleId,
            severity,
            sourceCode
        };
        this.messages.push(problem);
    }
    clean(fileUrl) {
        const file = url.format(fileUrl);
        _.remove(this.messages, (message) => {
            return message.resource === file;
        });
    }
    clear() {
        this.messages = [];
    }
    async notify() {
        await this.emitAsync('print', this.messages);
    }
    async executeOn(target) {
        const start = Date.now();
        debug(`Starting the analysis on ${target.href}`);
        await this.connector.collect(target);
        debug(`Total runtime ${Date.now() - start}`);
        return this.messages;
    }
    querySelectorAll(selector) {
        return this.connector.querySelectorAll(selector);
    }
    emitAsync(event, ...values) {
        const ignoredUrls = this.ignoredUrls.get('all');
        if (this.isIgnored(ignoredUrls, values[0].resource)) {
            return Promise.resolve([]);
        }
        return super.emitAsync(event, ...values);
    }
}
exports.Sonarwhal = Sonarwhal;
