"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fileType = require("file-type");
const isSvg = require("is-svg");
const mimeDB = require("mime-db");
const content_type_1 = require("content-type");
const debug_1 = require("./debug");
const misc_1 = require("./misc");
exports.getFileExtension = misc_1.getFileExtension;
const debug = debug_1.debug(__filename);
const getMediaTypeBasedOnFileExtension = (fileExtension) => {
    return fileExtension && Object.keys(mimeDB).find((key) => {
        return mimeDB[key].extensions && mimeDB[key].extensions.includes(fileExtension);
    }) || null;
};
const determineCharset = (originalCharset, mediaType) => {
    const charsetAliases = new Map([
        ['iso-8859-1', 'latin1']
    ]);
    const defaultCharset = charsetAliases.get(originalCharset) || originalCharset;
    const typeInfo = mimeDB[mediaType];
    let determinedCharset = typeInfo && misc_1.normalizeString(typeInfo.charset);
    if (defaultCharset && (determinedCharset === defaultCharset)) {
        return defaultCharset;
    }
    if (!isTextMediaType(mediaType)) {
        return null;
    }
    determinedCharset = determinedCharset || 'utf-8';
    return defaultCharset ? defaultCharset : determinedCharset;
};
const determineMediaTypeForScript = (element) => {
    const typeAttribute = misc_1.normalizeString(element.getAttribute('type'));
    const validJavaScriptMediaTypes = [
        'application/ecmascript',
        'application/javascript',
        'application/x-ecmascript',
        'application/x-javascript',
        'text/ecmascript',
        'text/javascript',
        'text/javascript1.0',
        'text/javascript1.1',
        'text/javascript1.2',
        'text/javascript1.3',
        'text/javascript1.4',
        'text/javascript1.5',
        'text/jscript',
        'text/livescript',
        'text/x-ecmascript',
        'text/x-javascript'
    ];
    if (!typeAttribute ||
        validJavaScriptMediaTypes.includes(typeAttribute) ||
        typeAttribute === 'module') {
        return 'text/javascript';
    }
    return null;
};
exports.determineMediaTypeForScript = determineMediaTypeForScript;
const determineMediaTypeBasedOnElement = (element) => {
    const nodeName = element && misc_1.normalizeString(element.nodeName);
    if (nodeName) {
        if (nodeName === 'script') {
            return determineMediaTypeForScript(element);
        }
        if (nodeName === 'link') {
            const relValue = element.getAttribute('rel');
            switch (relValue) {
                case 'stylesheet':
                    return 'text/css';
                case 'manifest':
                    return 'application/manifest+json';
            }
        }
    }
    return null;
};
const determineMediaTypeBasedOnFileExtension = (resource) => {
    const fileExtension = misc_1.getFileExtension(resource);
    if (!fileExtension) {
        return null;
    }
    switch (fileExtension) {
        case 'html':
        case 'htm':
            return 'text/html';
        case 'xhtml':
            return 'application/xhtml+xml';
        case 'js':
            return 'text/javascript';
        case 'css':
            return 'text/css';
        case 'ico':
            return 'image/x-icon';
        case 'webmanifest':
            return 'application/manifest+json';
        case 'jpeg':
        case 'jpg':
            return 'image/jpeg';
        case 'png':
            return 'image/png';
        case 'gif':
            return 'image/gif';
        case 'svg':
            return 'image/svg+xml';
        case 'webp':
            return 'image/webp';
        case 'woff2':
            return 'font/woff2';
        case 'woff':
            return 'font/woff';
        case 'ttf':
            return 'font/ttf';
        case 'otf':
            return 'font/otf';
        case 'xml':
            return 'text/xml';
    }
    return getMediaTypeBasedOnFileExtension(fileExtension);
};
exports.determineMediaTypeBasedOnFileExtension = determineMediaTypeBasedOnFileExtension;
const determineMediaTypeBasedOnFileName = (resource, rawContent) => {
    const fileName = misc_1.getFileName(resource);
    if (!fileName) {
        return null;
    }
    const configFileNameRegex = /^\.[a-z0-9]+rc$/i;
    if (!configFileNameRegex.test(fileName)) {
        return null;
    }
    try {
        JSON.parse(rawContent.toString());
    }
    catch (err) {
        return 'text/plain';
    }
    return 'text/json';
};
exports.determineMediaTypeBasedOnFileName = determineMediaTypeBasedOnFileName;
const determineMediaTypeBasedOnFileType = (rawContent) => {
    const detectedFileType = fileType(rawContent);
    if (detectedFileType) {
        return getMediaTypeBasedOnFileExtension(detectedFileType.ext);
    }
    if (rawContent && isSvg(rawContent)) {
        return 'image/svg+xml';
    }
    return null;
};
const getPreferedMediaType = (mediaType) => {
    switch (mediaType) {
        case 'application/xml':
            return 'text/xml';
        default:
            return mediaType;
    }
};
const parseContentTypeHeader = (headers) => {
    const contentTypeHeaderValue = misc_1.normalizeString(headers ? headers['content-type'] : null);
    if (contentTypeHeaderValue === null) {
        debug(`'content-type' header was not specified`);
        return null;
    }
    let contentType;
    try {
        if (contentTypeHeaderValue === '') {
            throw new TypeError('invalid media type');
        }
        contentType = content_type_1.parse(contentTypeHeaderValue);
    }
    catch (e) {
        debug(`'content-type' header value is invalid (${e.message})`);
        return null;
    }
    return contentType;
};
const getContentTypeData = (element, resource, headers, rawContent) => {
    let originalMediaType = null;
    let originalCharset = null;
    const contentType = parseContentTypeHeader(headers);
    if (contentType) {
        originalCharset = contentType.parameters.charset;
        originalMediaType = contentType.type;
    }
    let mediaType = determineMediaTypeBasedOnElement(element) ||
        determineMediaTypeBasedOnFileType(rawContent) ||
        determineMediaTypeBasedOnFileExtension(resource) ||
        determineMediaTypeBasedOnFileName(resource, rawContent) ||
        originalMediaType;
    mediaType = getPreferedMediaType(mediaType);
    const charset = determineCharset(originalCharset, mediaType);
    return {
        charset,
        mediaType
    };
};
exports.getContentTypeData = getContentTypeData;
const isTextMediaType = (mediaType) => {
    const textMediaTypes = [
        /application\/(?:javascript|json|x-javascript|xml)/i,
        /application\/.*\+(?:json|xml)/i,
        /image\/svg\+xml/i,
        /text\/.*/i
    ];
    if (textMediaTypes.some((regex) => {
        return regex.test(mediaType);
    })) {
        return true;
    }
    return false;
};
exports.isTextMediaType = isTextMediaType;
const getType = (mediaType) => {
    if (!mediaType) {
        return 'unknown';
    }
    if (mediaType.startsWith('image')) {
        return 'image';
    }
    if (mediaType.startsWith('font') || mediaType === 'application/vnd.ms-fontobject') {
        return 'font';
    }
    switch (mediaType) {
        case 'text/javascript':
            return 'script';
        case 'text/css':
            return 'css';
        case 'application/json':
        case 'text/json':
            return 'json';
        case 'application/manifest+json':
            return 'manifest';
        case 'text/html':
        case 'application/xhtml+xml':
            return 'html';
        case 'text/xml':
            return 'xml';
        case 'text/plain':
            return 'txt';
    }
    return 'unknown';
};
exports.getType = getType;
