"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const url_1 = require("url");
const util_1 = require("util");
const content_type_1 = require("content-type");
const shell = require("shelljs");
const stripBom = require("strip-bom");
const requireUncached = require("require-uncached");
const stripComments = require("strip-json-comments");
const requestAsync = require("request-promise");
exports.requestAsync = requestAsync;
const debug_1 = require("./debug");
const debug = debug_1.debug(__filename);
const processDir = process.cwd();
const readdir = fs.readdirSync;
const cutString = (txt, length = 50) => {
    if (txt.length <= length) {
        return txt;
    }
    const partialLength = Math.floor(length - 3) / 2;
    return `${txt.substring(0, partialLength)} … ${txt.substring(txt.length - partialLength, txt.length)}`;
};
exports.cutString = cutString;
const delay = (millisecs) => {
    return new Promise((resolve) => {
        setTimeout(resolve, millisecs);
    });
};
exports.delay = delay;
const getFileName = (resource) => {
    return path.basename(resource);
};
exports.getFileName = getFileName;
const getFileExtension = (resource) => {
    let url;
    try {
        url = new url_1.URL(resource);
    }
    catch (err) {
        return path.extname(resource).split('.')
            .pop();
    }
    return path.extname(url.pathname).split('.')
        .pop();
};
exports.getFileExtension = getFileExtension;
const getHeaderValueNormalized = (headers, headerName, defaultValue) => {
    return normalizeString(headers && headers[normalizeString(headerName)], defaultValue);
};
exports.getHeaderValueNormalized = getHeaderValueNormalized;
const hasProtocol = (resource, protocol) => {
    return new url_1.URL(resource).protocol === protocol;
};
exports.hasProtocol = hasProtocol;
const isDataURI = (resource) => {
    return hasProtocol(resource, 'data:');
};
exports.isDataURI = isDataURI;
const isLocalFile = (resource) => {
    return hasProtocol(resource, 'file:');
};
exports.isLocalFile = isLocalFile;
const isHTMLDocument = (targetURL, responseHeaders) => {
    if (isLocalFile(targetURL)) {
        return true;
    }
    const contentTypeHeaderValue = responseHeaders['content-type'];
    let mediaType;
    try {
        mediaType = content_type_1.parse(contentTypeHeaderValue).type;
    }
    catch (e) {
        return false;
    }
    return mediaType === 'text/html';
};
exports.isHTMLDocument = isHTMLDocument;
const isHTTP = (resource) => {
    return hasProtocol(resource, 'http:');
};
exports.isHTTP = isHTTP;
const isHTTPS = (resource) => {
    return hasProtocol(resource, 'https:');
};
exports.isHTTPS = isHTTPS;
const normalizeString = (value, defaultValue) => {
    if (typeof value === 'undefined' || value === null) {
        return typeof defaultValue !== 'undefined' ? defaultValue : null;
    }
    return value.toLowerCase().trim();
};
exports.normalizeString = normalizeString;
const protocolRegex = /([^:]*):.*/;
const isRegularProtocol = (uri) => {
    const normalizedUri = normalizeString(uri);
    const exec = protocolRegex.exec(normalizedUri);
    const protocol = exec ? exec[1] : null;
    if (![null, 'http', 'https'].includes(protocol)) {
        debug(`Ignore protocol: ${protocol}`);
        return false;
    }
    return true;
};
exports.isRegularProtocol = isRegularProtocol;
const normalizeStringByDelimiter = (value, delimiter) => {
    return normalizeString(value).replace(/[^a-z0-9]/gi, delimiter);
};
exports.normalizeStringByDelimiter = normalizeStringByDelimiter;
const isNormalizedIncluded = (source, included) => {
    return normalizeString(source).includes(normalizeString(included));
};
exports.isNormalizedIncluded = isNormalizedIncluded;
const toCamelCase = (value) => {
    return value.split('-').reduce((accu, w) => {
        if (!accu.length) {
            return w.toLocaleLowerCase();
        }
        let current = accu;
        current += w.length ? `${w.charAt(0).toUpperCase()}${w.substr(1).toLowerCase()}` : '';
        return current;
    }, '');
};
exports.toCamelCase = toCamelCase;
const toPascalCase = (value) => {
    return value.split('-').reduce((accu, w) => {
        let current = accu;
        current += w.length ? `${w.charAt(0).toUpperCase()}${w.substr(1).toLowerCase()}` : '';
        return current;
    }, '');
};
exports.toPascalCase = toPascalCase;
const readFile = (filePath) => {
    return stripBom(fs.readFileSync(filePath, 'utf8'));
};
exports.readFile = readFile;
const readFileAsync = async (filePath) => {
    const content = await util_1.promisify(fs.readFile)(filePath, 'utf8');
    return stripBom(content);
};
exports.readFileAsync = readFileAsync;
const requestJSONAsync = (uri, options) => {
    const params = Object.assign({
        json: true,
        uri
    }, options);
    return requestAsync(params);
};
exports.requestJSONAsync = requestJSONAsync;
const writeFileAsync = async (filePath, data) => {
    await util_1.promisify(fs.writeFile)(filePath, data, { encoding: 'utf8' });
};
exports.writeFileAsync = writeFileAsync;
const loadJSONFile = (filePath) => {
    debug(`Loading JSON file: ${filePath}`);
    return JSON.parse(stripComments(readFile(filePath)));
};
exports.loadJSONFile = loadJSONFile;
const loadJSFile = (filePath) => {
    debug(`Loading JS file: ${filePath}`);
    return requireUncached(filePath);
};
exports.loadJSFile = loadJSFile;
const findPackageRoot = (dirname = __dirname, fileToFind = 'package.json') => {
    const content = readdir(dirname);
    if (content.includes(fileToFind)) {
        return dirname;
    }
    const parentFolder = path.resolve(dirname, '..');
    if (parentFolder === dirname) {
        throw new Error('No package found');
    }
    return findPackageRoot(parentFolder, fileToFind);
};
exports.findPackageRoot = findPackageRoot;
const findNodeModulesRoot = (dirname = __dirname) => {
    const packageRoot = findPackageRoot(dirname);
    const nodeModulesPath = path.join(packageRoot, '..');
    if (nodeModulesPath.endsWith('node_modules')) {
        return nodeModulesPath;
    }
    return path.join(packageRoot, 'node_modules');
};
exports.findNodeModulesRoot = findNodeModulesRoot;
const hasAttributeWithValue = (element, nodeName, attribute, value) => {
    if (!element || element.nodeName.toLowerCase() !== nodeName.toLowerCase()) {
        return false;
    }
    const relAttribute = element.getAttribute(attribute);
    if (!relAttribute) {
        return false;
    }
    const rels = relAttribute.toLowerCase()
        .split(' ');
    return rels.some((rel) => {
        return rel === value;
    });
};
exports.hasAttributeWithValue = hasAttributeWithValue;
const isFile = (filePath) => {
    return shell.test('-f', filePath);
};
exports.isFile = isFile;
const isDirectory = (directoryPath) => {
    return shell.test('-d', directoryPath);
};
exports.isDirectory = isDirectory;
const pathExists = (pathString) => {
    return shell.test('-e', pathString);
};
exports.pathExists = pathExists;
const isOfficial = async () => {
    try {
        const pkg = JSON.parse(await readFileAsync(path.join(findPackageRoot(processDir), 'package.json')));
        return pkg.name === '@sonarwhal/monorepo';
    }
    catch (e) {
        return false;
    }
};
exports.isOfficial = isOfficial;
const getPackage = (pathString) => {
    return require(`${pathString}/package.json`);
};
exports.getPackage = getPackage;
const getSonarwhalPackage = () => {
    return require(path.join(__dirname, '../../../../package.json'));
};
exports.getSonarwhalPackage = getSonarwhalPackage;
const toLowerCaseKeys = (obj) => {
    const entries = Object.entries(obj);
    return entries.reduce((lowerCased, [key, value]) => {
        lowerCased[key.toLowerCase()] = value;
        return lowerCased;
    }, {});
};
exports.toLowerCaseKeys = toLowerCaseKeys;
