"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path = require("path");
const npmRegistryFetch = require("npm-registry-fetch");
const ora = require("ora");
const debug_1 = require("./debug");
const logger = require("./logging");
const misc_1 = require("./misc");
const debug = debug_1.debug(__filename);
const install = (command) => {
    return new Promise((resolve, reject) => {
        const npmInstall = child_process_1.spawn(command, [], { shell: true });
        let errors = '';
        npmInstall.stderr.setEncoding('utf8');
        npmInstall.stderr.on('data', (data) => {
            errors += data;
        });
        npmInstall.on('error', (err) => {
            return reject(err);
        });
        npmInstall.on('exit', (code) => {
            if (code !== 0) {
                return reject(errors);
            }
            return resolve(true);
        });
    });
};
exports.installPackages = async (packages) => {
    let global = false;
    let isDev = false;
    let packagePath;
    const currentWorkingDir = process.cwd();
    const isWindows = process.platform === 'win32';
    let command = `npm install ${packages.join(' ')}`;
    if (packages.length === 0) {
        return Promise.resolve(true);
    }
    try {
        packagePath = misc_1.findPackageRoot(currentWorkingDir);
        global = packagePath !== currentWorkingDir;
    }
    catch (error) {
        global = true;
    }
    const spinner = ora({ spinner: 'line' });
    try {
        if (!global) {
            const jsonContent = misc_1.loadJSONFile(path.join(packagePath, 'package.json'));
            isDev = jsonContent.devDependencies && jsonContent.devDependencies.hasOwnProperty('sonarwhal');
        }
        command += global ? ' -g' : '';
        command += isDev ? ' --save-dev' : '';
        debug(`Running command ${command}`);
        logger.log('Installing packages...');
        spinner.text = `Running command: ${command}`;
        spinner.start();
        await install(command);
        spinner.succeed();
        return true;
    }
    catch (err) {
        debug(err);
        spinner.text = `Error executing "${command}"`;
        spinner.fail();
        if ((err.message || err).includes('404')) {
            logger.error(`One or more of the packages don't exist`);
        }
        else {
            logger.error(`Try executing:
    ${!isWindows && global ? 'sudo ' : ''}${command}
            manually to install all the packages.`);
        }
        return false;
    }
};
const filterPackages = (packages, initTerm) => {
    return packages.filter((pkg) => {
        return pkg.name.startsWith(initTerm);
    });
};
const getPackages = (result) => {
    return result.objects.map((obj) => {
        return obj.package;
    });
};
const generateSearchQuery = (searchTerm, from, size = 100) => {
    return `/-/v1/search?text=${searchTerm}&size=${size}${from ? `&from=${from}` : ''}`;
};
exports.search = async (searchTerm) => {
    const result = await npmRegistryFetch.json(generateSearchQuery(searchTerm));
    let total = getPackages(result);
    while (result.total > total.length) {
        const r = await npmRegistryFetch.json(generateSearchQuery(searchTerm, total.length));
        total = total.concat(getPackages(r));
    }
    return total;
};
exports.getOfficialPackages = async (type) => {
    const rules = await exports.search(`@sonarwhal/${type}`);
    return filterPackages(rules, `@sonarwhal/${type}`);
};
exports.getUnnoficialPackages = async (type) => {
    const rules = await exports.search(`sonarwhal-${type}`);
    return filterPackages(rules, `sonarwhal-${type}`);
};
