"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const globby = require("globby");
const semver = require("semver");
const misc_1 = require("../utils/misc");
const debug_1 = require("../utils/debug");
const resourcetype_1 = require("../enums/resourcetype");
const debug = debug_1.debug(__filename);
const SONARWHAL_ROOT = misc_1.findPackageRoot();
const NODE_MODULES_ROOT = misc_1.findNodeModulesRoot();
const moduleNameRegex = /[^']*'([^']*)'/g;
const resources = new Map();
const resourceIds = new Map();
const isVersionValid = (resourcePath) => {
    try {
        const pkg = misc_1.getPackage(resourcePath);
        const sonarwhalPkg = misc_1.getSonarwhalPackage();
        return semver.satisfies(sonarwhalPkg.version, pkg.peerDependencies.sonarwhal);
    }
    catch (e) {
        debug(e);
        return true;
    }
};
exports.getCoreResources = (type) => {
    if (resourceIds.has(type)) {
        return resourceIds.get(type);
    }
    const resourcesFiles = globby.sync(`dist/src/lib/${type}s/**/*.js`, { cwd: SONARWHAL_ROOT });
    const ids = resourcesFiles.reduce((list, resourceFile) => {
        const resourceName = path.basename(resourceFile, '.js');
        if (path.dirname(resourceFile).includes(resourceName)) {
            list.push(resourceName);
        }
        return list;
    }, []);
    resourceIds.set(type, ids);
    return ids;
};
const hasMultipleResources = (resource, type) => {
    switch (type) {
        case resourcetype_1.ResourceType.rule:
            return !resource.meta;
        default:
            return false;
    }
};
exports.getInstalledResources = (type) => {
    const installedType = `installed-${type}`;
    if (resourceIds.has(installedType)) {
        return resourceIds.get(installedType);
    }
    const resourcesFiles = globby.sync(`${NODE_MODULES_ROOT}/@sonarwhal/${type}-*/package.json`);
    const ids = resourcesFiles.reduce((list, resourceFile) => {
        const resource = require(path.dirname(resourceFile));
        const packageName = JSON.parse(misc_1.readFile(resourceFile)).name;
        const resourceName = packageName.substr(packageName.lastIndexOf('/') + 1).replace(`${type}-`, '');
        if (!hasMultipleResources(resource, type)) {
            list.push(resourceName);
        }
        else {
            const rules = Object.entries(resource);
            if (rules.length === 1 && resource[resourceName]) {
                list.push(resourceName);
            }
            else {
                for (const [key] of rules) {
                    list.push(`${resourceName}/${key}`);
                }
            }
        }
        return list;
    }, []);
    resourceIds.set(installedType, ids);
    return ids;
};
exports.tryToLoadFrom = (resourcePath) => {
    let builder = null;
    try {
        const resource = require(resourcePath);
        builder = resource.default || resource;
    }
    catch (e) {
        debug(`Can't require ${resourcePath}`);
        if (e.code === 'MODULE_NOT_FOUND') {
            const exec = moduleNameRegex.exec(e.message);
            const moduleName = exec ? exec[1] : null;
            if (!moduleName || moduleName === resourcePath) {
                return null;
            }
            throw new Error(`Module ${moduleName} not found when loading ${resourcePath}`);
        }
        throw e;
    }
    return builder;
};
const getResource = (source, type, name) => {
    const resource = exports.tryToLoadFrom(source);
    if (!resource) {
        return null;
    }
    if (!hasMultipleResources(resource, type)) {
        return resource;
    }
    for (const [key, value] of Object.entries(resource)) {
        if (key === name) {
            return value.default || value;
        }
    }
    return null;
};
const generateConfigPathsToResources = (configurations, name, type) => {
    return configurations.reduce((total, configuration) => {
        const basePackagePaths = ['@sonarwhal/configuration-', 'sonarwhal-configuration-'];
        let result = total;
        for (const basePackagePath of basePackagePaths) {
            const packageName = `${basePackagePath}${configuration}`;
            try {
                const packagePath = path.dirname(require.resolve(packageName));
                const resourcePackages = globby.sync(`node_modules/{@sonarwhal/,sonarwhal-}${type}-${name}/package.json`, { absolute: true, cwd: packagePath }).map((pkg) => {
                    return path.dirname(pkg);
                });
                result = result.concat(resourcePackages);
            }
            catch (err) {
                debug(`Package ${packageName} not found`);
            }
        }
        return result;
    }, []);
};
exports.loadResource = (name, type, configurations = [], verifyVersion = false) => {
    debug(`Searching ${name}…`);
    const nameSplitted = name.split('/');
    const packageName = nameSplitted[0];
    const resourceName = nameSplitted[1] || packageName;
    const key = `${type}-${name}`;
    if (resources.has(key) && !verifyVersion) {
        return resources.get(key);
    }
    const configPathsToResources = generateConfigPathsToResources(configurations, packageName, type);
    const currentProcessDir = process.cwd();
    const sources = [
        `@sonarwhal/${type}-${packageName}`,
        `sonarwhal-${type}-${packageName}`,
        path.normalize(`${SONARWHAL_ROOT}/dist/src/lib/${type}s/${packageName}/${packageName}.js`),
        path.normalize(currentProcessDir)
    ].concat(configPathsToResources);
    let resource;
    let isValid = true;
    sources.some((source) => {
        const res = getResource(source, type, resourceName);
        if (res) {
            debug(`${name} found in ${source}`);
            if (source === currentProcessDir) {
                try {
                    const packageConfig = misc_1.getPackage(source);
                    if (!misc_1.isNormalizedIncluded(packageConfig.name, packageName)) {
                        return false;
                    }
                }
                catch (e) {
                    return false;
                }
            }
            if (verifyVersion && !isVersionValid(source)) {
                debug(`Resource ${name} isn't compatible with current sonarwhal version`);
                isValid = false;
                return false;
            }
            isValid = true;
            resource = res;
        }
        return resource;
    });
    if (!isValid) {
        throw new Error(`Resource ${name} isn't compatible with current sonarwhal version`);
    }
    if (!resource) {
        debug(`Resource ${name} not found`);
        throw new Error(`Resource ${name} not found`);
    }
    resources.set(key, resource);
    return resource;
};
const loadListOfResources = (list = [], type, configurations = []) => {
    const missing = [];
    const incompatible = [];
    const items = Array.isArray(list) ?
        list :
        Object.keys(list);
    const loadedResources = items.reduce((loaded, resourceId) => {
        try {
            const resource = exports.loadResource(resourceId, type, configurations, true);
            loaded.push(resource);
        }
        catch (e) {
            if (e.message.includes(`isn't compatible`)) {
                incompatible.push(`${type}-${resourceId}`);
            }
            else {
                missing.push(`${type}-${resourceId}`);
            }
        }
        return loaded;
    }, []);
    return {
        incompatible,
        missing,
        resources: loadedResources
    };
};
exports.loadRule = (ruleId, configurations) => {
    return exports.loadResource(ruleId, resourcetype_1.ResourceType.rule, configurations);
};
exports.loadConfiguration = (configurationId) => {
    return exports.loadResource(configurationId, resourcetype_1.ResourceType.configuration);
};
exports.loadResources = (config) => {
    let connector = null;
    try {
        connector = exports.loadResource(config.connector.name, resourcetype_1.ResourceType.connector, config.extends, true);
    }
    catch (e) {
        console.error(e);
    }
    const { incompatible: incompatibleRules, resources: rules, missing: missingRules } = loadListOfResources(config.rules, resourcetype_1.ResourceType.rule, config.extends);
    const { incompatible: incompatibleParsers, resources: parsers, missing: missingParsers } = loadListOfResources(config.parsers, resourcetype_1.ResourceType.parser, config.extends);
    const { incompatible: incompatibleFormatters, resources: formatters, missing: missingFormatters } = loadListOfResources(config.formatters, resourcetype_1.ResourceType.formatter, config.extends);
    const missing = [].concat(missingRules, missingParsers, missingFormatters);
    const incompatible = [].concat(incompatibleFormatters, incompatibleParsers, incompatibleRules);
    if (!connector) {
        missing.push(config.connector);
    }
    return {
        connector,
        formatters,
        incompatible,
        missing,
        parsers,
        rules
    };
};
