"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ajv = require("ajv");
const _ = require("lodash");
const validator = new ajv({
    $data: true,
    allErrors: true,
    logger: false,
    schemaId: 'id',
    useDefaults: true,
    verbose: true
});
validator.addMetaSchema(require('ajv/lib/refs/json-schema-draft-04.json'));
var ErrorKeyword;
(function (ErrorKeyword) {
    ErrorKeyword["additionalProperties"] = "additionalProperties";
    ErrorKeyword["anyOf"] = "anyOf";
    ErrorKeyword["enum"] = "enum";
    ErrorKeyword["pattern"] = "pattern";
    ErrorKeyword["type"] = "type";
})(ErrorKeyword || (ErrorKeyword = {}));
const generateError = (type, action) => {
    return (error, errors) => {
        if (error.keyword !== type) {
            return null;
        }
        const property = error.dataPath.substr(1);
        return action(error, property, errors);
    };
};
const generateAdditionalPropertiesError = generateError(ErrorKeyword.additionalProperties, (error, property) => {
    const additionalProperty = error.params.additionalProperty;
    return `${property ? `'${property}' ` : ''}${property ? error.message : `${error.message[0].toLocaleUpperCase()}${error.message.substr(1)}`}. Additional property found '${additionalProperty}'.`;
});
const generateEnumError = generateError(ErrorKeyword.enum, (error, property) => {
    const allowedValues = error.params.allowedValues;
    return `'${property}' ${error.message} '${allowedValues.join(', ')}'. Value found '${error.data}'`;
});
const generatePatternError = generateError(ErrorKeyword.pattern, (error, property) => {
    return `'${property}' ${error.message.replace(/"/g, '\'')}. Value found '${error.data}'`;
});
const generateTypeError = generateError(ErrorKeyword.type, (error, property) => {
    return `'${property}' ${error.message.replace(/"/g, '\'')}.`;
});
const generateAnyOfError = generateError(ErrorKeyword.anyOf, (error, property, errors) => {
    const otherErrors = _.without(errors, error);
    const results = _.map(otherErrors, (otherError) => {
        return generate(otherError);
    });
    return results.join(' or ');
});
const errorGenerators = [generateAdditionalPropertiesError, generateEnumError, generatePatternError, generateTypeError, generateAnyOfError];
const generate = (error, errors) => {
    return _.reduce(errorGenerators, (message, generator) => {
        const newErrorMessage = generator(error, errors);
        if (newErrorMessage) {
            return newErrorMessage;
        }
        return message;
    }, error.message);
};
const prettify = (errors) => {
    const grouped = _.groupBy(errors, 'dataPath');
    const result = _.reduce(grouped, (allMessages, groupErrors) => {
        _.forEach(groupErrors, (error) => {
            allMessages.push(generate(error, groupErrors));
        });
        return allMessages;
    }, []);
    return result;
};
exports.validate = (schema, json) => {
    const data = _.cloneDeep(json);
    const validateFunction = validator.compile(schema);
    const valid = validateFunction(data);
    return {
        data,
        errors: validateFunction.errors,
        prettifiedErrors: prettify(validateFunction.errors),
        valid
    };
};
