"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const ava_1 = require("ava");
const retry = require("async-retry");
const connectors_1 = require("./connectors");
const test_server_1 = require("./test-server");
const resourceLoader = require("../../src/lib/utils/resource-loader");
const sonarwhal_1 = require("../../src/lib/sonarwhal");
const config_1 = require("../../src/lib/config");
const get_as_uri_1 = require("../../src/lib/utils/get-as-uri");
const localhostRegex = /(http|https):\/\/localhost[:]*[0-9]*\//g;
const createConfig = (id, connector, opts) => {
    const rules = {};
    if (opts && opts.ruleOptions) {
        rules[id] = ['error', opts.ruleOptions];
    }
    else {
        rules[id] = 'error';
    }
    const config = {
        browserslist: opts && opts.browserslist || [],
        connector: {
            name: connector,
            options: {}
        },
        parsers: opts && opts.parsers || [],
        rules
    };
    if (connector === 'jsdom') {
        config.connector.options = {
            rejectUnauthorized: false,
            strictSSL: false
        };
    }
    if (connector === 'chrome') {
        config.connector.options = { overrideInvalidCert: true };
    }
    return config_1.SonarwhalConfig.fromConfig(config);
};
const validateResults = (t, results, reports) => {
    const server = t.context.server || {};
    if (!reports) {
        return t.is(results.length, 0, `Received results is ${JSON.stringify(results, null, 2)}`);
    }
    if (results.length === 0) {
        return t.fail(`No results found, should be ${reports.length}`);
    }
    if (results.length !== reports.length) {
        return t.fail(`Result count is ${results.length}, should be ${reports.length}`);
    }
    return reports.forEach((report, index) => {
        if (server.port) {
            t.is(results[index].message, report.message.replace(localhostRegex, `$1://localhost:${server.port}/`), `Different message`);
        }
        else {
            t.is(results[index].message, report.message, `Different message`);
        }
        if (report.position) {
            t.is(results[index].location.column, report.position.column, `Different column`);
            t.is(results[index].location.line, report.position.line, `Different line`);
        }
    });
};
exports.testRule = (ruleId, ruleTests, configs = {}) => {
    ava_1.test.beforeEach(async (t) => {
        if (typeof t.context.server !== 'undefined') {
            return;
        }
        t.context.server = test_server_1.createServer(configs.https);
        await t.context.server.start();
    });
    ava_1.test.afterEach.always(async (t) => {
        await t.context.server.stop();
    });
    const createConnector = async (t, ruleTest, connector, attemp) => {
        const { server } = t.context;
        const { serverConfig } = ruleTest;
        if (ruleTest.before) {
            await ruleTest.before();
        }
        const config = createConfig(ruleId, connector, configs);
        const resources = resourceLoader.loadResources(config);
        const sonarwhal = new sonarwhal_1.Sonarwhal(config, resources);
        if (attemp === 1 && serverConfig) {
            server.configure(serverConfig);
        }
        return sonarwhal;
    };
    const stopConnector = async (ruleTest, connector) => {
        if (ruleTest.after) {
            await ruleTest.after();
        }
        await connector.close();
    };
    const runRule = (t, ruleTest, connector) => {
        return retry(async (bail, attemp) => {
            if (attemp > 1) {
                console.log(`[${connector}] ${ruleTest.name} - try ${attemp}`);
            }
            try {
                const { server } = t.context;
                const { serverUrl, reports } = ruleTest;
                const target = serverUrl ? serverUrl : `${configs.https ? 'https' : 'http'}://localhost:${server.port}/`;
                const sonarwhal = await createConnector(t, ruleTest, connector, attemp);
                const results = await sonarwhal.executeOn(new url_1.URL(target));
                await stopConnector(ruleTest, sonarwhal);
                return validateResults(t, results, reports);
            }
            catch (e) {
                console.error(e);
                return false;
            }
        }, {
            minTimeout: 10000,
            retries: 3
        });
    };
    const Rule = resourceLoader.loadRule(ruleId, []);
    connectors_1.ids.forEach((connector) => {
        if (!Rule.meta.ignoredConnectors || !Rule.meta.ignoredConnectors.includes(connector)) {
            ruleTests.forEach((ruleTest) => {
                let runner;
                runner = configs['serial'] ? ava_1.test.serial : ava_1.test;
                if (configs.ignoredConnectors && configs.ignoredConnectors.includes(connector)) {
                    runner = ava_1.test.skip;
                }
                runner(`[${connector}] ${ruleTest.name}`, runRule, ruleTest, connector);
            });
        }
    });
};
exports.testLocalRule = (ruleId, ruleTests, configs = {}) => {
    const Rule = resourceLoader.loadRule(ruleId, []);
    if (Rule.meta.ignoredConnectors && Rule.meta.ignoredConnectors.includes('local')) {
        return;
    }
    const sonarwhalConfig = createConfig(ruleId, 'local', configs);
    const resources = resourceLoader.loadResources(sonarwhalConfig);
    let runner;
    runner = configs.serial ? ava_1.test.serial : ava_1.test;
    if (configs.ignoredConnectors && configs.ignoredConnectors.includes('local')) {
        runner = ava_1.test.skip;
    }
    const runRule = (t, ruleTest, connector) => {
        return retry(async (bail, attemp) => {
            if (attemp > 1) {
                console.log(`[${connector}] ${ruleTest.name} - try ${attemp}`);
            }
            try {
                if (ruleTest.before) {
                    await ruleTest.before(t);
                }
                const sonarwhal = new sonarwhal_1.Sonarwhal(sonarwhalConfig, resources);
                const results = await sonarwhal.executeOn(get_as_uri_1.getAsUri(ruleTest.path));
                await sonarwhal.close();
                if (ruleTest.after) {
                    await ruleTest.after(t);
                }
                return validateResults(t, results, ruleTest.reports);
            }
            catch (e) {
                console.error(e);
                return false;
            }
        }, {
            minTimeout: 10000,
            retries: 3
        });
    };
    ruleTests.forEach((ruleTest) => {
        runner(`[local] ${ruleTest.name}`, runRule, ruleTest, 'local');
    });
};
