"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const http = require("http");
const https = require("https");
const path = require("path");
const _ = require("lodash");
const express = require("express");
const onHeaders = require("on-headers");
const misc_1 = require("../../src/lib/utils/misc");
const maxPort = 65535;
class Server {
    constructor(isHTTPS) {
        this._port = _.random(3000, 65000);
        this._app = express();
        this._app.disable('x-powered-by');
        this._app.use((req, res, next) => {
            res.setHeader('Cache-Control', 'no-cache');
            next();
        });
        this._isHTTPS = isHTTPS;
    }
    updateLocalhost(html) {
        return html.replace(/\/\/localhost\//g, `//localhost:${this._port}/`);
    }
    handleHeaders(res, headers) {
        onHeaders(res, () => {
            Object.entries(headers).forEach(([header, value]) => {
                if (value !== null) {
                    res.setHeader(header, value);
                }
                else {
                    res.removeHeader(header);
                }
            });
        });
    }
    getContent(value) {
        if (typeof value === 'string') {
            return this.updateLocalhost(value);
        }
        else if (value && typeof value.content !== 'undefined') {
            return typeof value.content === 'string' ? this.updateLocalhost(value.content) : value.content;
        }
        return '';
    }
    getNumberOfMatches(req, requestConditions) {
        const headers = requestConditions.request && requestConditions.request.headers;
        if (!headers) {
            return 0;
        }
        let numberOfMatches = 0;
        for (const [header, value] of Object.entries(headers)) {
            const headerValue = misc_1.getHeaderValueNormalized(req.headers, header);
            if ((headerValue !== misc_1.normalizeString(value)) || (!headerValue && (value === null))) {
                return 0;
            }
            numberOfMatches++;
        }
        return numberOfMatches;
    }
    getValue(req, config) {
        let bestNumberOfMatches = 1;
        let bestMatch = null;
        for (const [key, value] of Object.entries(config)) {
            let requestConditions;
            try {
                requestConditions = JSON.parse(key);
                const newNumberOfMatches = this.getNumberOfMatches(req, requestConditions);
                if (newNumberOfMatches >= bestNumberOfMatches) {
                    bestMatch = value;
                    bestNumberOfMatches = newNumberOfMatches;
                }
            }
            catch (e) {
            }
        }
        return bestMatch;
    }
    isConditionalConfig(configuration) {
        try {
            return typeof JSON.parse(Object.entries(configuration)[0][0]) === 'object';
        }
        catch (e) {
        }
        return false;
    }
    normalizeConfig(configuration) {
        const config = {};
        for (const [k, v] of Object.entries(configuration)) {
            for (const [key, value] of Object.entries(v)) {
                config[key] = Object.assign({}, config[key], { [k]: value });
            }
        }
        return config;
    }
    configure(configuration) {
        let customFavicon = false;
        if (typeof configuration === 'string') {
            this._app.get('/', (req, res) => {
                res.send(configuration);
            });
            return;
        }
        const conditionalConfig = this.isConditionalConfig(configuration);
        const config = conditionalConfig ? this.normalizeConfig(configuration) : configuration;
        _.forEach(config, (val, key) => {
            customFavicon = customFavicon || key === '/favicon.ico';
            this._app.get(key, (req, res) => {
                const value = conditionalConfig ? this.getValue(req, val) : val;
                const content = this.getContent(value);
                if (value === null) {
                    res.redirect(301, 'test://fa.il');
                    return;
                }
                const redirects = [301, 302, 303, 307, 308];
                if (value && redirects.includes(value.status)) {
                    res.redirect(value.status, content);
                    return;
                }
                if (value && value.status) {
                    res.status(value.status);
                }
                if (value && value.headers) {
                    this.handleHeaders(res, value.headers);
                }
                if (content !== 'timeout') {
                    res.send(content);
                }
            });
        });
        if (!customFavicon) {
            this._app.get('/favicon.ico', (req, res) => {
                res.status(200);
                res.setHeader('Content-Length', '0');
                res.setHeader('Content-Type', 'image/x-icon');
                res.end();
            });
        }
    }
    start() {
        return new Promise(async (resolve, reject) => {
            let options;
            if (this._isHTTPS) {
                options = {
                    cert: await fs.readFile(path.join(__dirname, 'fixture/server.crt'), 'utf8'),
                    key: await fs.readFile(path.join(__dirname, 'fixture/server.key'), 'utf8')
                };
                this._server = https.createServer(options, this._app);
            }
            else {
                this._server = http.createServer(this._app);
            }
            this._server.on('error', (err) => {
                console.error(err.code);
                if (err.code === 'EADDRINUSE' || err.code === 'EACCES') {
                    setImmediate(() => {
                        this._port++;
                        if (this._port > maxPort) {
                            this._port = _.random(3000, 65000);
                        }
                        this._server.close();
                        this._server.listen(this._port);
                    });
                }
                else {
                    reject(err);
                }
            });
            this._server.once('listening', () => {
                resolve();
            });
            this._server.listen(this._port);
        });
    }
    stop() {
        this._server.close();
    }
    get port() {
        return this._port;
    }
}
exports.Server = Server;
exports.createServer = (isHTTPS) => {
    return new Server(isHTTPS);
};
